import random as r

r.seed(10)

for _ in range(100):
    i = r.randint(1, 10)
    assert i <= 10
    assert i >= 1
    i = r.random()
    assert 0.0 <= i <= 1.0
    i = r.uniform(3.0, 9.5)
    assert 3.0 <= i <= 9.5

a = [1, 2, 3, 4]
r.shuffle(a)

for i in range(10):
    assert r.choice(a) in a

for i in range(10):
    assert r.choice(tuple(a)) in a

for i in range(10):
    assert r.choice('hello') in 'hello'

for i in range(10):
    assert r.randint(1, 1) == 1

# test choices
x = (1,)
res = r.choices(x, k=4)
assert (res == [1, 1, 1, 1]), res

w = (1, 2, 3)
assert r.choices([1, 2, 3], (0.0, 0.0, 0.5)) == [3]

try:
    r.choices([1, 2, 3], (0.0, 0.0, 0.5, 0.5))
    exit(1)
except ValueError:
    pass

try:
    r.choices([])
    exit(1)
except IndexError:
    pass

seq = [1, 2, 3, 4]
weights = [0.1, 0.2, 0.2, 0.5]
k = 1000
res = r.choices(seq, weights, k=k)
assert len(res) == k and isinstance(res, list)

max_error = 0.03
for i in range(len(seq)):
    actual_w = res.count(seq[i]) / k
    assert abs(actual_w - weights[i]) < max_error

# test seed
from random import randint, seed
seed(7)
a = randint(1, 100)
b = randint(-2**60, 1)
c = randint(50, 100)

assert (a, b, c) == (16, -418020281577586157, 76)

seed(7)
assert a == randint(1, 100)
assert b == randint(-2**60, 1)
assert c == randint(50, 100)

import random
assert random.Random(7).randint(1, 100) == a



import json
from ffigen import Library, set_vmath_converter

with open('../3rd/raylib/parser/output/raylib_api.json') as f:
    data = json.load(f)

lib = Library.from_raylib(data)
set_vmath_converter('Vector2', 'vec2')
set_vmath_converter('Vector3', 'vec3')

lib.build(
    includes=['raylib.h'],
    glue_dir='../src',
    stub_dir='../include/typings'
)



emptyDict = dict()
assert len(emptyDict) == 0
tinydict = {'Name': 'Tom', 'Age': 7, 'Class': 'First'}
assert tinydict['Name'] == 'Tom';assert tinydict['Age'] == 7
tinydict['Age'] = 8;tinydict['School'] = "aaa"
assert tinydict['Age'] == 8;assert tinydict['School'] == "aaa"
del tinydict['Name']

assert repr(tinydict) == "{'Age': 8, 'Class': 'First', 'School': 'aaa'}"
assert len(tinydict) == 3
tinydict.clear()
assert len(tinydict) == 0

dict1 = {'user':'circle','num':[1,2,3]}
dict2 = dict1.copy()

for k,v in dict1.items():
    assert dict2[k] == v
for t in dict1.items():
    assert t == ('user', 'circle') or t == ('num', [1, 2, 3])

tinydict = {'Name': 'circle', 'Age': 7}
tinydict2 = {'Sex': 'female' }
assert len(tinydict) == 2
assert len(tinydict2) == 1
tinydict.update(tinydict2)
updated_dict = {'Name': 'circle', 'Age': 7, 'Sex': 'female'}
for k,v in tinydict.items():
    assert updated_dict[k] == v
assert len(tinydict) == 3
assert tinydict == updated_dict

dishes = {'eggs': 2, 'sausage': 1, 'bacon': 1, 'spam': 500}
# dict is now ordered
assert list(dishes.keys()) == ['eggs', 'sausage', 'bacon', 'spam']
assert list(dishes.values()) == [2, 1, 1, 500]

d={1:"a",2:"b",3:"c"}
result=[]
for k,v in d.items():
    result.append(k)
    result.append(v)
assert len(result) == 6

del d[2]
assert len(d) == 2
assert list(d.keys()) == [1, 3]
assert list(d.values()) == ['a', 'c']
del d[1]
del d[3]
assert len(d) == 0

# test __eq__
d1 = {1:2, 3:4}
d2 = {3:4, 1:2}
d3 = {1:2, 3:4, 5:6}
assert d1 == d2
assert d1 != d3

a = dict([(1, 2), (3, 4)])
assert a == {1: 2, 3: 4}

assert a.pop(1) == 2
assert a == {3: 4}
assert a.pop(3) == 4
assert a == {}

a = {}
for i in range(1000):
    a[i] = i
assert len(a) == 1000
for i in range(1000):
    del a[i]
assert len(a) == 0

a = {'g': 0}

a['ball_3'] = 0
a['ball_4'] = 0
assert list(a.keys()) == ['g', 'ball_3', 'ball_4']
del a['ball_3']
assert list(a.keys()) == ['g', 'ball_4']
del a['ball_4']
assert list(a.keys()) == ['g',]
del a['g']
assert len(a) == 0

# ultra test!!
a = {'0': 0, '1': 1}
for i in range(2, 1000):
    a[str(i)] = i
    del a[str(i - 2)]
    assert a[str(i - 1)] == i - 1

a = {'0': 0, '1': 1}
b = ['0', '1']

# dict delete test
data = []
j = 6
for i in range(65535):
    j = ((j*5+1) % 65535)
    data.append(str(j))

for i in range(len(data)):
    z = data[i]
    a[z] = i
    b.append(z)
    if i % 3 == 0:
        y = b.pop()
        del a[y]

a = {1: 2, 3: 4}
assert a.pop(1) == 2

assert a.pop(1, None) is None

# test getitem
d = {}
for i in range(-1000, 1000):
    d[i] = i
    assert d[i] == i

e = {}
for i in range(-10000, 10000, 3):
    e[i] = i
    assert e[i] == i

# test iter
d = {'1': 1, 222: 2, '333': 3}
assert list(d) == ['1', 222, '333']
assert list(d.keys()) == ['1', 222, '333']
assert list(d.values()) == [1, 2, 3]
assert list(d.items()) == [('1', 1), (222, 2), ('333', 3)]

# test del
n = 2 ** 17
a = {}
for i in range(n):
    a[str(i)] = i
for i in range(n):
    del a[str(i)]
assert len(a) == 0

# test popitem
n = 2 ** 17
a = {}
for i in range(n):
    a[str(i)] = i
for i in range(n):
    k, v = a.popitem()
    assert k == str(n - 1 - i)
    assert v == n - 1 - i
assert len(a) == 0

# test del with int keys
if 0:
    n = 2 ** 17
    a = {}
    for i in range(n):
        a[i] = i
    for i in range(n):
        del a[i]
    assert len(a) == 0

#######################

# namedict delete test
class A: pass
a = A()
b = ['0', '1']

for i in range(len(data)):
    z = data[i]
    setattr(a, str(z), i)
    b.append(z)
    if i % 3 == 0:
        y = b.pop()
        delattr(a, y)



import os, sys
assert sys.platform == 'darwin', sys.platform

if len(sys.argv) == 2:
    build_dir = sys.argv[1]
    output_dir = sys.argv[1]
elif len(sys.argv) == 3:
    build_dir = sys.argv[1]
    output_dir = sys.argv[2]
else:
    print('Usage: python merge_built_libraries.py <build_dir> [output_dir]')
    exit(1)

assert os.path.exists(build_dir), build_dir
assert os.path.exists(output_dir), output_dir

archives = []

# get all .a files in build/3rd recursive
for root, dirs, files in os.walk(build_dir):
    for file in files:
        if file.endswith('.a') and file.startswith('lib'):
            archives.append(os.path.join(root, file))

print('Merging the following static libraries:')
for archive in archives:
    print('- ' + archive)

# libtool -static -o libpocketpy.a
output_archive = os.path.join(output_dir, 'libpocketpy.a')
os.system('libtool -static -o {} {}'.format(output_archive, ' '.join(archives)))



try:
    import os
except ImportError:
    exit(0)

import sys
is_pyc = sys.argv[0].endswith('.pyc')

if is_pyc:
    os.chdir('tmp/tests')
else:
    os.chdir('tests')

assert os.getcwd().endswith('tests')

import test1
assert test1.add(1, 2) == 13

from test2.a.g import get_value, A
assert get_value() == '123'
assert (A.__module__ == 'test2.a.g'), A.__module__

import test2.a.g
assert test2.a.g.get_value() == '123'

from test2.utils import get_value_2
assert get_value_2() == '123'

from test3.a.b import value
assert value == 1

import test3.a.b as test3_ab
assert test3_ab.value == 1

from test2.utils import r
assert r.__name__ == 'test2.utils.r'
assert r.__package__ == 'test2.utils'

def f():
    import math as m
    assert m.pi > 3

    from test3.a.b import value
    assert value == 1

f()

from math import *
assert pi > 3

from math import (pi, pow, sin, cos)

from math import (
    pi,
    pow,
    sin,
    cos
)

assert __import__('math').pi > 3

# test reload (dummy)
if not is_pyc:
    import importlib
    importlib.reload(test2.a)

    



import numba
from typing import List

@numba.jit(nopython=True)
def sieve_of_eratosthenes(n: int) -> List[int]:
    assert n >= 2
    is_prime = [True] * (n + 1)
    is_prime[0] = is_prime[1] = False  # 0 和 1 不是素数
    
    for start in range(2, int(n**0.5) + 1):
        if is_prime[start]:
            for multiple in range(start*start, n + 1, start):
                is_prime[multiple] = False
    
    primes = [num for num, prime in enumerate(is_prime) if prime]
    return primes

all_primes = sieve_of_eratosthenes(2**30)
print(len(all_primes), all_primes[:10], all_primes[-10:])

index = 3
caps = [all_primes[index]]

while True:
    for i in range(index+1, len(all_primes)):
        last_cap = caps[-1]
        if last_cap < 1000:
            min_cap = last_cap * 2
        else:
            min_cap = last_cap * 1.5
        if all_primes[i] >= min_cap:
            caps.append(all_primes[i])
            index = i
            break
    else:
        break

print('-'*20)
print(caps)

print('switch(cap) {')
for i in range(len(caps)-1):
    print(f'    case {caps[i]}:', f'return {caps[i+1]};')
print('    default: c11__unreachable();')
print('}')


import array2d
from vmath import vec2i


default = 0
a = array2d.chunked_array2d(16, default, auto_add_chunk=False)
assert a.chunk_size == 16

a.add_chunk(vec2i(1, 1), 5.0)
a[vec2i(16, 16)] = 16
a[vec2i(17, 16)] = 15
assert a[vec2i(16, 16)] == 16
assert a[vec2i(17, 16)] == 15
assert a[vec2i(17, 20)] == default

a1, _ = a.world_to_chunk(vec2i(16, 16))

assert a.get_context(vec2i(1,1)) == 5.0
assert a.remove_chunk(a1)



import gc

gc.setup_debug_callback(print)

gc.collect()
def create_garbage():
    a = [(1,2) for i in range(20000)]
    return a

create_garbage()
gc.collect()

create_garbage()
create_garbage()
create_garbage()


from time import localtime
import operator

class timedelta:
    def __init__(self, days=0, seconds=0):
        self.days = days
        self.seconds = seconds

    def __repr__(self):
        return f"datetime.timedelta(days={self.days}, seconds={self.seconds})"

    def __eq__(self, other) -> bool:
        if not isinstance(other, timedelta):
            return NotImplemented
        return (self.days, self.seconds) == (other.days, other.seconds)

    def __ne__(self, other) -> bool:
        if not isinstance(other, timedelta):
            return NotImplemented
        return (self.days, self.seconds) != (other.days, other.seconds)


class date:
    def __init__(self, year: int, month: int, day: int):
        self.year = year
        self.month = month
        self.day = day

    @staticmethod
    def today():
        t = localtime()
        return date(t.tm_year, t.tm_mon, t.tm_mday)
    
    def __cmp(self, other, op):
        if not isinstance(other, date):
            return NotImplemented
        if self.year != other.year:
            return op(self.year, other.year)
        if self.month != other.month:
            return op(self.month, other.month)
        return op(self.day, other.day)

    def __eq__(self, other) -> bool:
        return self.__cmp(other, operator.eq)
    
    def __ne__(self, other) -> bool:
        return self.__cmp(other, operator.ne)

    def __lt__(self, other: 'date') -> bool:
        return self.__cmp(other, operator.lt)

    def __le__(self, other: 'date') -> bool:
        return self.__cmp(other, operator.le)

    def __gt__(self, other: 'date') -> bool:
        return self.__cmp(other, operator.gt)

    def __ge__(self, other: 'date') -> bool:
        return self.__cmp(other, operator.ge)

    def __str__(self):
        return f"{self.year}-{self.month:02}-{self.day:02}"

    def __repr__(self):
        return f"datetime.date({self.year}, {self.month}, {self.day})"


class datetime(date):
    def __init__(self, year: int, month: int, day: int, hour: int, minute: int, second: int):
        super().__init__(year, month, day)
        # Validate and set hour, minute, and second
        if not 0 <= hour <= 23:
            raise ValueError("Hour must be between 0 and 23")
        self.hour = hour
        if not 0 <= minute <= 59:
            raise ValueError("Minute must be between 0 and 59")
        self.minute = minute
        if not 0 <= second <= 59:
            raise ValueError("Second must be between 0 and 59")
        self.second = second

    def date(self) -> date:
        return date(self.year, self.month, self.day)

    @staticmethod
    def now():
        t = localtime()
        tm_sec = t.tm_sec
        if tm_sec == 60:
            tm_sec = 59
        return datetime(t.tm_year, t.tm_mon, t.tm_mday, t.tm_hour, t.tm_min, tm_sec)

    def __str__(self):
        return f"{self.year}-{self.month:02}-{self.day:02} {self.hour:02}:{self.minute:02}:{self.second:02}"

    def __repr__(self):
        return f"datetime.datetime({self.year}, {self.month}, {self.day}, {self.hour}, {self.minute}, {self.second})"

    def __cmp(self, other, op):
        if not isinstance(other, datetime):
            return NotImplemented
        if self.year != other.year:
            return op(self.year, other.year)
        if self.month != other.month:
            return op(self.month, other.month)
        if self.day != other.day:
            return op(self.day, other.day)
        if self.hour != other.hour:
            return op(self.hour, other.hour)
        if self.minute != other.minute:
            return op(self.minute, other.minute)
        return op(self.second, other.second)

    def __eq__(self, other) -> bool:
        return self.__cmp(other, operator.eq)
    
    def __ne__(self, other) -> bool:
        return self.__cmp(other, operator.ne)
    
    def __lt__(self, other) -> bool:
        return self.__cmp(other, operator.lt)
    
    def __le__(self, other) -> bool:
        return self.__cmp(other, operator.le)
    
    def __gt__(self, other) -> bool:
        return self.__cmp(other, operator.gt)
    
    def __ge__(self, other) -> bool:
        return self.__cmp(other, operator.ge)





from array2d import array2d
from vmath import vec2i

def exit_on_error():
    raise KeyboardInterrupt

# test error args for __init__
try:
    a = array2d(0, 0)
    exit_on_error()
except ValueError:
    pass

# test callable constructor
a = array2d[tuple[int, int]](2, 4, lambda pos: (pos.x, pos.y))

assert a.width == a.n_cols == 2
assert a.height == a.n_rows == 4
assert a.shape == vec2i(2, 4)
assert a.numel == 8
assert a.tolist() == [
    [(0, 0), (1, 0)],
    [(0, 1), (1, 1)],
    [(0, 2), (1, 2)],
    [(0, 3), (1, 3)]]

assert a[0, :].tolist() == [[(0, 0)], [(0, 1)], [(0, 2)], [(0, 3)]]
assert a[1, :].tolist() == [[(1, 0)], [(1, 1)], [(1, 2)], [(1, 3)]]
assert a[:, 0].tolist() == [[(0, 0), (1, 0)]]
assert a[:, -1].tolist() == [[(0, 3), (1, 3)]]

# test is_valid
assert a.is_valid(0, 0) and a.is_valid(vec2i(0, 0))
assert a.is_valid(1, 3) and a.is_valid(vec2i(1, 3))
assert not a.is_valid(2, 0) and not a.is_valid(vec2i(2, 0))
assert not a.is_valid(0, 4) and not a.is_valid(vec2i(0, 4))
assert not a.is_valid(-1, 0) and not a.is_valid(vec2i(-1, 0))
assert not a.is_valid(0, -1) and not a.is_valid(vec2i(0, -1))

# test get
assert a.get(0, 0, -1) == (0, 0)
assert a.get(1, 3) == (1, 3)
assert a.get(2, 0) is None
assert a.get(0, 4, 'S') == 'S'

# test index
assert a.index((0, 0)) == vec2i(0, 0)
assert a.index((1, 3)) == vec2i(1, 3)

# test __getitem__
assert a[0, 0] == (0, 0)
assert a[1, 3] == (1, 3)
try:
    a[2, 0]
    exit_on_error()
except IndexError:
    pass

# test __setitem__
a = array2d[int](2, 4, default=0)
a[0, 0] = 5
assert a[0, 0] == 5
a[1, 3] = 6
assert a[1, 3] == 6
try:
    a[0, -1] = 7
    exit_on_error()
except IndexError:
    pass

# test tolist
a_list = [[5, 0], [0, 0], [0, 0], [0, 6]]
assert a_list == a.tolist()

# test __eq__
x = array2d(2, 4, default=0)
b = array2d(2, 4, default=0)
assert (x == b).all()

b[0, 0] = 1
assert (x != b).any()

# test __repr__
assert repr(a) == f'array2d(2, 4)'

# test map
c = a.map(lambda x: x + 1)
assert c.tolist() == [[6, 1], [1, 1], [1, 1], [1, 7]]
assert a.tolist() == [[5, 0], [0, 0], [0, 0], [0, 6]]
assert c.width == c.n_cols == 2
assert c.height == c.n_rows == 4
assert c.numel == 8

# test copy
d = c.copy()
assert (d == c).all() and d is not c

# test fill_
d[:, :] = -3    # d.fill_(-3)
assert (d == array2d(2, 4, default=-3)).all()

# test apply
d.apply(lambda x: x + 3)
assert (d == array2d(2, 4, default=0)).all()

# test copy_
a[:, :] = d
assert (a == d).all() and a is not d
x = array2d(2, 4, default=0)
x[:, :] = d
assert (x == d).all() and x is not d

# test alive_neighbors
a = array2d[int](3, 3, default=0)
a[1, 1] = 1
"""     Moore    von Neumann
0 0 0   1 1 1    0 1 0
0 1 0   1 0 1    1 0 1
0 0 0   1 1 1    0 1 0
"""
moore_result = array2d(3, 3, default=1)
moore_result[1, 1] = 0

von_neumann_result = array2d(3, 3, default=0)
von_neumann_result[0, 1] = von_neumann_result[1, 0] = von_neumann_result[1, 2] = von_neumann_result[2, 1] = 1

_0 = a.count_neighbors(1, 'Moore')
assert _0 == moore_result
_1 = a.count_neighbors(1, 'von Neumann')
assert _1 == von_neumann_result

MOORE_KERNEL = array2d[int].fromlist([[1, 1, 1], [1, 0, 1], [1, 1, 1]])
VON_NEUMANN_KERNEL = array2d.fromlist([[0, 1, 0], [1, 0, 1], [0, 1, 0]])
moore_conv_result = a.convolve(MOORE_KERNEL, 0)
assert (moore_conv_result == moore_result).all()
von_neumann_conv_result = a.convolve(VON_NEUMANN_KERNEL, 0)
assert (von_neumann_conv_result == von_neumann_result).all()

# test slice get
a = array2d(5, 5, default=0)
b = array2d(3, 2, default=1)

assert a[1:4, 1:4] == array2d(3, 3, default=0)
assert a[1:4, 1:3] == array2d(3, 2, default=0)
assert (a[1:4, 1:3] != b).any()
a[1:4, 1:3] = b
assert (a[1:4, 1:3] == b).all()
"""
0 0 0 0 0
0 1 1 1 0
0 1 1 1 0
0 0 0 0 0
0 0 0 0 0
"""
assert a.count(1) == 3*2

assert a.get_bounding_rect(1) == (1, 1, 3, 2)
assert a.get_bounding_rect(0) == (0, 0, 5, 5)

try:
    a.get_bounding_rect(2)
    exit_on_error()
except ValueError:
    pass

a = array2d(3, 2, default='?')
# int/float/str/bool/None

for value in [0, 0.0, '0', False, None]:
    a[0:2, 0:1] = value
    assert a[2, 1] == '?'
    assert a[0, 0] == value

a[:, :] = 3
assert a == array2d(3, 2, default=3)

try:
    a[:, :] = array2d(1, 1)
    exit_on_error()
except ValueError:
    pass

# test __iter__
a = array2d(3, 4, default=1)
for xy, val in a:
    assert a[xy] == x

# test convolve
a = array2d[int].fromlist([[1, 0, 2, 4, 0], [3, 1, 0, 5, 1]])
"""
1 0 2 4 0
3 1 0 5 1
"""
assert a.tolist() == [[1, 0, 2, 4, 0], [3, 1, 0, 5, 1]]

kernel = array2d[int](3, 3, default=1)
res = a.convolve(kernel, -1)
"""
0 4 9 9 5
0 4 9 9 5
"""
assert res.tolist() == [[0, 4, 9, 9, 5], [0, 4, 9, 9, 5]]

mask = res == 9
assert mask.tolist() == [
    [False, False, True, True, False],
    [False, False, True, True, False]
    ]
assert res[mask] == [9, 9, 9, 9]

mask = res != 9
assert mask.tolist() == [
    [True, True, False, False, True],
    [True, True, False, False, True]
    ]
assert res[mask] == [0, 4, 5, 0, 4, 5]
res[mask] = -1
assert res.tolist() == [[-1, -1, 9, 9, -1], [-1, -1, 9, 9, -1]]

# test get_connected_components
a = array2d[int].fromlist([
    [1, 1, 0, 1],
    [0, 2, 2, 1],
    [0, 1, 1, 1],
    [1, 0, 0, 0],
])
vis, cnt = a.get_connected_components(1, 'von Neumann')
assert vis == [
    [1, 1, 0, 2],
    [0, 0, 0, 2],
    [0, 2, 2, 2],
    [3, 0, 0, 0]
    ]
assert cnt == 3
vis, cnt = a.get_connected_components(1, 'Moore')
assert vis == [
    [1, 1, 0, 2],
    [0, 0, 0, 2],
    [0, 2, 2, 2],
    [2, 0, 0, 0]
    ]
assert cnt == 2
vis, cnt = a.get_connected_components(2, 'von Neumann')
assert cnt == 1
vis, cnt = a.get_connected_components(0, 'Moore')
assert cnt == 2

# test zip_with
a = array2d[int].fromlist([[1, 2], [3, 4]])
b = array2d[int].fromlist([[5, 6], [7, 8]])
c = a.zip_with(b, lambda x, y: x + y)
assert c.tolist() == [[6, 8], [10, 12]]

# test magic op
a = array2d[int].fromlist([[1, 2], [3, 4]])
assert (a <= 2).tolist() == [[True, True], [False, False]]
assert (a < 2).tolist() == [[True, False], [False, False]]
assert (a >= 2).tolist() == [[False, True], [True, True]]
assert (a > 2).tolist() == [[False, False], [True, True]]
assert (a == 2).tolist() == [[False, True], [False, False]]
assert (a != 2).tolist() == [[True, False], [True, True]]
assert (a + 1).tolist() == [[2, 3], [4, 5]]
assert (a - 1).tolist() == [[0, 1], [2, 3]]
assert (a * 2).tolist() == [[2, 4], [6, 8]]
assert (a / 1).tolist() == [[1.0, 2.0], [3.0, 4.0]]
assert (a // 2).tolist() == [[0, 1], [1, 2]]
assert (a % 2).tolist() == [[1, 0], [1, 0]]
assert (a ** 2).tolist() == [[1, 4], [9, 16]]

a = array2d[bool].fromlist([[True, False], [False, True]])
assert (a & True).tolist() == [[True, False], [False, True]]
assert (a | True).tolist() == [[True, True], [True, True]]
assert (a ^ True).tolist() == [[False, True], [True, False]]

b = array2d[bool].fromlist([[True, True], [False, False]])
assert (a & b).tolist() == [[True, False], [False, False]]
assert (a | b).tolist() == [[True, True], [False, True]]
assert (a ^ b).tolist() == [[False, True], [False, True]]
assert (~a).tolist() == [[False, True], [True, False]]
assert (~b).tolist() == [[False, False], [True, True]]

# stackoverflow bug due to recursive mark-and-sweep
# class Cell:
#     neighbors: list['Cell']

# cells: array2d[Cell] = array2d(192, 108, default=Cell)
# OutOfBounds = Cell()
# for x, y, cell in cells:
#     cell.neighbors = [
#         cells.get(x-1, y-1, OutOfBounds),
#         cells.get(x  , y-1, OutOfBounds),
#         cells.get(x+1, y-1, OutOfBounds),
#         cells.get(x-1, y  , OutOfBounds),
#         cells.get(x+1, y  , OutOfBounds),
#         cells.get(x  , y+1, OutOfBounds),
#         cells.get(x+1, y+1, OutOfBounds),
#     ]

# import gc
# gc.collect()



from array2d import array2d
from vmath import color32, vec2i

class Image:
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...

    def __new__(cls, width: int, height: int) -> "Image": ...

    @staticmethod
    def from_bytes(data: bytes) -> "Image": ...
    @staticmethod
    def from_file(path: str) -> "Image": ...

    def setpixel(self, x: int, y: int, color: color32) -> None: ...
    def getpixel(self, x: int, y: int) -> color32: ...
    def clear(self, color: color32) -> None: ...

    def paste(self, src_img: "Image", src_pos: vec2i, dst_pos: vec2i, width: int, height: int, fg: color32, bg: color32) -> None: ...

    def to_png_bytes(self) -> bytes: ...
    def to_png_file(self, path: str) -> int: ...
    def to_rgb565_file(self, path: str) -> int: ...


def loads(data: bytes) -> array2d[color32]: ...
def dumps(image: array2d[color32]) -> bytes: ...



try:
    import lz4
except ImportError:
    print('lz4 is not enabled, skipping test...')
    exit()

def test(data: bytes):
    compressed = lz4.compress(data)
    decompressed = lz4.decompress(compressed)
    assert data == decompressed
    if len(data) == 0:
        return 0
    return len(compressed) / len(data)

test(b'')
test(b'hello world')

import random

def gen_data():
    values = []
    for i in range(random.randint(0, 10000)):
        values.append(i % 256)
    return bytes(values)


for i in range(100):
    ratio = test(gen_data())
    # print(f'compression ratio: {ratio:.2f}')

# test 64MB random data (require 1GB list[int] buffer)
rnd = [random.randint(0, 255) for _ in range(1024*1024*1024//16)]
test(bytes(rnd))



# test basic get/set
import random
random.seed(7)

a = {str(i): i for i in range(100)}
a['existed'] = 0
a['missed'] = 0

for i in range(1000000):
    key = str(random.randint(-100, 100))
    if key in a:
        a['existed'] += 1
    else:
        a['missed'] += 1

existed = a['existed']
missed = a['missed']

assert abs(existed - missed) < 10000

# rnd = random.Random(7)
# assert rnd.randint(1, 100) == 16
# assert rnd.randint(1, 100) == 93
# assert rnd.randint(1, 100) == 22


a = 0

a += 2
assert a == 2

a -= 1
assert a == 1

a *= 2
assert a == 2

a //= 2
assert a == 1

a |= 0xff
assert a == 0xff

a &= 0x0f
assert a == 0x0f

a = 8

a %= 3
assert a == 2

a ^= 0xf0
assert a == 242

# incremental set
class A: pass

for i in range(ord('a'), ord('z')+1):
    setattr(A, chr(i), i)

assert A.a == ord('a')
assert A.z == ord('z')

assert ord('测') == 27979

try:
    assert ord('测试')
    print("Should not reach here")
    exit(1)
except TypeError:
    pass

try:
    assert ord('12')
    print("Should not reach here")
    exit(1)
except TypeError:
    pass


def Linear(t: float) -> float: ...
def InSine(t: float) -> float: ...
def OutSine(t: float) -> float: ...
def InOutSine(t: float) -> float: ...
def InQuad(t: float) -> float: ...
def OutQuad(t: float) -> float: ...
def InOutQuad(t: float) -> float: ...
def InCubic(t: float) -> float: ...
def OutCubic(t: float) -> float: ...
def InOutCubic(t: float) -> float: ...
def InQuart(t: float) -> float: ...
def OutQuart(t: float) -> float: ...
def InOutQuart(t: float) -> float: ...
def InQuint(t: float) -> float: ...
def OutQuint(t: float) -> float: ...
def InOutQuint(t: float) -> float: ...
def InExpo(t: float) -> float: ...
def OutExpo(t: float) -> float: ...
def InOutExpo(t: float) -> float: ...
def InCirc(t: float) -> float: ...
def OutCirc(t: float) -> float: ...
def InOutCirc(t: float) -> float: ...
def InBack(t: float) -> float: ...
def OutBack(t: float) -> float: ...
def InOutBack(t: float) -> float: ...
def InElastic(t: float) -> float: ...
def OutElastic(t: float) -> float: ...
def InOutElastic(t: float) -> float: ...
def InBounce(t: float) -> float: ...
def OutBounce(t: float) -> float: ...
def InOutBounce(t: float) -> float: ...



from .writer import Writer
from .types import C_INT_TYPES, C_FLOAT_TYPES, C_BOOL_TYPES, C_STRING_TYPES, VMATH_TYPES

class Converter:
    def __init__(self, T: str):
        self.T = T
    def c2py(self, w: Writer, out: str, expr: str):
        raise NotImplementedError
    def py2c(self, w: Writer, out: str, expr: str):
        raise NotImplementedError
    @property
    def py_T(self) -> str:
        raise NotImplementedError
    
    def is_const(self):
        return self.T.startswith('const ') or '[' in self.T
    
class _SimpleConverter(Converter):
    def c2py(self, w: Writer, out: str, expr: str):
        w.write(f'py_new{self.py_T}({out}, {expr});')
    def py2c(self, w: Writer, out: str, expr: str):
        w.write(f'if(!py_check{self.py_T}({expr})) return false;')
        w.write(f'{out} = py_to{self.py_T}({expr});')
    
class IntConverter(_SimpleConverter):
    @property
    def py_T(self) -> str:
        return 'int'
    
class BoolConverter(_SimpleConverter):
    @property
    def py_T(self) -> str:
        return 'bool'
    
class StringConverter(_SimpleConverter):
    @property
    def py_T(self) -> str:
        return 'str'
    
class FloatConverter(Converter):
    def c2py(self, w: Writer, out: str, expr: str):
        w.write(f'py_newfloat({out}, {expr});')
    def py2c(self, w: Writer, out: str, expr: str):
        if self.T == 'float':
            w.write(f'if(!py_castfloat32({expr}, &{out})) return false;')
        else:
            w.write(f'if(!py_castfloat({expr}, &{out})) return false;')

    @property
    def py_T(self) -> str:
        return 'float'

class PointerConverter(Converter):
    def c2py(self, w: Writer, out: str, expr: str):
        w.write(f'py_newint({out}, (py_i64){expr});')
    def py2c(self, w: Writer, out: str, expr: str):
        w.write(f'if(!py_checkint({expr})) return false;')
        w.write(f'{out} = ({self.T})py_toint({expr});')

    @property
    def py_T(self) -> str:
        return 'intptr'

class StructConverter(Converter):
    def __init__(self, T: str, type_index: str | None):
        super().__init__(T)
        if type_index is None:
            type_index = f'tp_user_{T}'
        self.type_index = type_index
    def c2py(self, w: Writer, out: str, expr: str):
        w.write('do {')
        w.indent()
        w.write(f'{self.T}* ud = py_newobject({out}, {self.type_index}, 0, sizeof({self.T}));')
        w.write(f'*ud = {expr};')
        w.dedent()
        w.write('} while(0);')
    def py2c(self, w: Writer, out: str, expr: str):
        w.write('do {')
        w.indent()
        w.write(f'if(!py_checktype({expr}, {self.type_index})) return false;')
        w.write(f'{out} = *({self.T}*)py_touserdata({expr});')
        w.dedent()
        w.write('} while(0);')

    @property
    def py_T(self) -> str:
        return self.T
    
class EnumConverter(Converter):
    def __init__(self, T: str):
        super().__init__(T)
    def c2py(self, w: Writer, out: str, expr: str):
        w.write(f'py_newint({out}, (py_i64){expr});')
    def py2c(self, w: Writer, out: str, expr: str):
        w.write(f'if(!py_checkint({expr})) return false;')
        w.write(f'{out} = ({self.T})py_toint({expr});')

    @property
    def py_T(self) -> str:
        return 'int'
    
class VoidConverter(Converter):
    def c2py(self, w: Writer, out: str, expr: str):
        w.write(f'py_newnone({out});')
    def py2c(self, w: Writer, out: str, expr: str):
        # raise NotImplementedError
        w.write(f'? // VoidConverter.py2c is not implemented')
    
    @property
    def py_T(self) -> str:
        return 'None'
    
class BuiltinVMathConverter(Converter):
    def __init__(self, T: str, py_builtin_T: str):
        super().__init__(T)
        self.py_builtin_T = py_builtin_T

    def c2py(self, w: Writer, out: str, expr: str):
        w.write(f'py_new{self.py_builtin_T}({out}, *(c11_{self.py_T}*)(&{expr}));')

    def py2c(self, w: Writer, out: str, expr: str):
        w.write('do {')
        w.indent()
        w.write(f'if(!py_checktype({expr}, tp_{self.py_T})) return false;')
        w.write(f'c11_{self.py_T} tmp = py_to{self.py_builtin_T}({expr});')
        # w.write(f'memcpy(&{out}, &tmp, sizeof(c11_{self.py_T}));')
        w.write(f'{out} = *({self.T}*)(&tmp);')
        w.dedent()
        w.write('} while(0);')
    
    @property
    def py_T(self) -> str:
        return self.py_builtin_T


_CONVERTERS: dict[str, Converter] = {}

for t in C_INT_TYPES:
    _CONVERTERS[t] = IntConverter(t)
for t in C_FLOAT_TYPES:
    _CONVERTERS[t] = FloatConverter(t)
for t in C_BOOL_TYPES:
    _CONVERTERS[t] = BoolConverter(t)
for t in C_STRING_TYPES:
    _CONVERTERS[t] = StringConverter(t)
for t in VMATH_TYPES:
    _CONVERTERS[t] = BuiltinVMathConverter(f'c11_{t}', t)

_CONVERTERS['void'] = VoidConverter('void')
_CONVERTERS['c11_array2d'] = StructConverter('c11_array2d', 'tp_array2d')

def is_vmath_type(T: str) -> bool:
    cvt = _CONVERTERS.get(T)
    if cvt is None:
        return False
    return isinstance(cvt, BuiltinVMathConverter)

_has_vmath_converter = False
def set_vmath_converter(T: str, py_T: str):
    global _has_vmath_converter
    assert py_T in VMATH_TYPES
    _CONVERTERS[T] = BuiltinVMathConverter(T, py_T)
    _has_vmath_converter = True

def has_vmath_converter() -> bool:
    return _has_vmath_converter

def set_enum_converter(T: str):
    _CONVERTERS[T] = EnumConverter(T)

def get_converter(T: str) -> Converter:
    if T in _CONVERTERS:
        return _CONVERTERS[T]
    if T.endswith('*'):
        return PointerConverter(T)
    if '[' in T:
        return PointerConverter(T)
    cvt = _CONVERTERS.get(T)
    if cvt is None:
        return StructConverter(T, None)
    return cvt



from cmath import isclose, sqrt, nan, inf, nanj, infj, log
import math

c1 = complex(3, 4)
c2 = complex(2, 4.5)

assert isclose(c1 + 5, complex(8, 4))
assert isclose(c1 + c2, complex(5, 8.5))
assert isclose(c1 - c2, complex(1, -0.5))
assert isclose(c1*4, complex(12, 16))
assert isclose(c1*c2, complex(-12, 21.5))
assert isclose(c2/c1, complex(0.96, 0.22))
assert isclose(c2**2, complex(-16.25, 18))

assert 1+2j == complex(1, 2) == 2j+1

assert isclose(1+2j + 3+4j, 4+6j)

assert isclose(1+2j - 3+4j, -2+6j)

assert (1+2j).real == 1
assert (1+2j).imag == 2

assert isclose((1+2j)*(3+4j), -5+10j)
assert isclose((1+2j)*3, 3+6j)

assert isclose((1+2j)**2, -3+4j)

assert (1+2j).conjugate() == 1-2j

res = sqrt(1+2j)
assert isclose(res, 1.272019649514069+0.7861513777574233j)

assert {1+2j: 1}[1+2j] == 1

assert repr(1+2j) == '(1.0+2.0j)'
assert repr(1+0j) == '(1.0+0.0j)'
assert repr(-1-3j) == '(-1.0-3.0j)'
assert repr(1-3j) == '(1.0-3.0j)'


assert repr(math.nan) == repr(nan) == 'nan'
assert repr(-math.nan) == repr(-nan) == 'nan'
assert repr(math.inf) == repr(inf) == 'inf'
assert repr(-math.inf) == repr(-inf) == '-inf'
assert repr(nanj) == '(0.0+nanj)', nanj
assert repr(-nanj) == '(-0.0+nanj)', -nanj
assert repr(infj) == '(0.0+infj)', infj
assert repr(-infj) == '(-0.0-infj)', -infj

assert math.log(1) == 0.0
assert isclose(log(10+5j), 2.4141568686511508+0.4636476090008061j)



import os
os.chdir('benchmarks')

import json

_2489KB = 'WorldMap_GridVania_layout.ldtk'
_1093KB = 'WorldMap_Free_layout.ldtk'
_339KB = 'Typical_2D_platformer_example.ldtk'

with open(f'res/{_2489KB}', 'r') as f:
    json_content = f.read()

data: dict = json.loads(json_content)
assert isinstance(data, dict)

# serialize and deserialize
dumped: str = json.dumps(data)
loaded: dict = json.loads(dumped)
assert len(data) == len(loaded)
assert data == loaded



import unicodedata
from tqdm import trange
from typing import Literal

info = []

for i in trange(0x110000):
    char = chr(i)
    category = unicodedata.category(char)
    east_asian_width = unicodedata.east_asian_width(char)
    info.append((i, category, east_asian_width))

def merge(index: Literal[1, 2], filter):
    # index = 1, category
    # index = 2, east_asian_width
    result: list[tuple[int, int, str]] = []
    last_value = None
    last_start = None
    for i in range(len(info)):
        value = info[i][index]
        if value != last_value:
            if last_value is not None:
                result.append((last_start, i - 1, last_value))
            last_value = value
            last_start = i
    if last_value is not None:
        result.append((last_start, len(info) - 1, last_value))
    return [x for x in result if filter(x[2])]

df_category = merge(1, lambda x: x == 'Lo')
df_east_asian_width = merge(2, lambda x: x != 'N')

def to_c11(ranges, name, with_value=True):
    with open(f'{name}.c', 'wt', encoding='utf-8', newline='\n') as f:
        f.write(f'const static c11_u32_range {name}[] = {{\n')
        for start, end, value in ranges:
            if with_value:
                f.write(f'    {{ {start}, {end}, "{value}\\0" }},\n')
            else:
                f.write(f'    {{ {start}, {end} }},\n')
        f.write(f'}};\n')

to_c11(df_category, 'kLoRanges', with_value=False)
to_c11(df_east_asian_width, 'kEastAsianWidthRanges', with_value=True)





# test type hints

def f(x: int) -> int:
    return x + 1

def g(x: int, y: int) -> int:
    return x + y

def h(x: int, y):
    return x + y

def i(x, y: int):
    return x + y

# test type hints with default values

def f(x: int = 1) -> int:
    return x + 1

def g(x: int = 1, y: int = 2) -> int:
    return x + y

def h(x: int = 1, y = 2):
    return x + y

def i(x = 1, y: int = 2):
    return x + y

# test type hints with *args

def f(x: int, *args) -> int:
    return x + len(args)

def g(x: int, y: int, *args) -> int:
    return x + y + len(args)

def h(x: int, y, *args):
    return x + y + len(args)

def i(x, y: int, *args):
    return x + y + len(args)

def j(x, y: int, *args: str) -> int:
    return x + y + len(args)

x: int = 1
y: 'str' = '2'

x: 'list[int]' = [1, 2, 3]
y: 'list[str]' = ['1', '2', '3']

def g(x: 'list[int]', y: 'list[str]') -> 'list[int]':
    return x + y

def z(x: float):
    x: int = 1
    y: 'str' = '2'


import pcpp
import pycparser
from ffigen import Library, set_vmath_converter, set_enum_converters
from ffigen.meta import Header
import os

path = '../3rd/box2d/include/box2d/box2d.h'
code = pcpp.CmdPreprocessor([None, path, '-o', 'tmp.h', '--line-directive', '-I', 'libc_include', '-I', '../3rd/box2d/include'])

ast = pycparser.parse_file('tmp.h')
os.remove('tmp.h')

header = Header()
header.build(ast)

header.remove_types({'b2Timer', 'b2DebugDraw'})
header.remove_functions({'b2CreateTimer', 'b2Hash', 'b2DefaultDebugDraw'})

lib = Library.from_header('box2d', header)

set_vmath_converter('b2Vec2', 'vec2')
set_vmath_converter('b2Vec3', 'vec3')

set_enum_converters([enum.name for enum in lib.enums])

lib.build(
    includes=['box2d/box2d.h'],
    glue_dir='../src',
    stub_dir='../include/typings'
)



import pcpp
import pycparser
from ffigen.library import Library
from ffigen.converters import set_vmath_converter, set_enum_converter
from ffigen.meta import Header
import os

file_dir = os.path.dirname(os.path.abspath(__file__))

path = '3rd/periphery/include/periphery.h'
code = pcpp.CmdPreprocessor([None, path, '-o', 'tmp.h', '-I', os.path.join(file_dir, 'libc_include')])

mapping = {
    'enum serial_parity parity': 'serial_parity_t parity',
}
# remap tmp.h
with open('tmp.h', 'r') as f:
    content = f.read()
for k, v in mapping.items():
    content = content.replace(k, v)
with open('tmp.h', 'w') as f:
    f.write(content)

ast = pycparser.parse_file('tmp.h')
os.remove('tmp.h')

header = Header()
header.build(ast)

lib = Library.from_header('periphery', header)

lib.build(
    includes=['periphery.h'],
    glue_dir='3rd/periphery/src',
    stub_dir='include/typings',
)



class A:
    def __init__(self, a, b):
        self.a = a
        self.b = b

    def add(self):
        return self.a + self.b

    def sub(self):
        return self.a - self.b
    
a = A(1, 2)
assert a.add() == 3
assert a.sub() == -1

assert A.__base__ is object

class B(A):
    def __init__(self, a, b, c):
        super(B, self).__init__(a, b)
        self.c = c

    def add(self):
        return self.a + self.b + self.c

    def sub(self):
        return self.a - self.b - self.c

assert B.__base__ is A    

b = B(1, 2, 3)
assert b.add() == 6
assert b.sub() == -4

class C(B):
    def __init__(self, a, b, c, d):
        super(C, self).__init__(a, b, c)
        self.d = d

    def add(self):
        return self.a + self.b + self.c + self.d

    def sub(self):
        return self.a - self.b - self.c - self.d
    
assert C.__base__ is B

c = C(1, 2, 3, 4)
assert c.add() == 10
assert c.sub() == -8

class D(C):
    def __init__(self, a, b, c, d, e):
        super(D, self).__init__(a, b, c, d)
        self.e = e

    def add(self):
        return super(D, self).add() + self.e

    def sub(self):
        return super(D, self).sub() - self.e
    
assert D.__base__ is C

d = D(1, 2, 3, 4, 5)
assert d.add() == 15
assert d.sub() == -13

assert isinstance(1, int)
assert isinstance(1, object)
assert isinstance(C, type)
assert isinstance(C, object)
assert isinstance(d, object)
assert isinstance(d, C)
assert isinstance(d, B)
assert isinstance(d, A)
assert isinstance(object, object)
assert isinstance(type, object)

assert isinstance(1, (float, int))
assert isinstance(1, (float, object))
assert not isinstance(1, (float, str))
assert isinstance(object, (int, type, float))
assert not isinstance(object, (int, float, str))

try:
    isinstance(1, (1, 2))
    exit(1)
except TypeError:
    pass

try:
    isinstance(1, 1)
    exit(1)
except TypeError:
    pass

class A:
    a = 1
    b = 2

assert A.a == 1
assert A.b == 2

class B(A):
    b = 3
    c = 4

assert B.b == 3
assert B.c == 4

assert B.a == 1

class A:
    x = 1
    x = x + 1
    y = 1
    z = x + y

assert A.x == 2
assert A.y == 1
assert A.z == 3

class MyClass:
    a = 1,2,3
    b, c = 1, 2
    d = b + c

    def __init__(self, m, n) -> None:
        self.m: int = m
        self.n: float = n

assert MyClass.a == (1, 2, 3)
assert MyClass.b == 1
assert MyClass.c == 2
assert MyClass.d == 3

assert MyClass(1, 2).m == 1

class E: pass
try:
    E(1,2,3)
    exit(1)
except TypeError:
    pass

class E1:
    def __new__(cls, a, b):
        o = object.__new__(cls)
        o.a = a
        o.b = b
        return o

    def sum(self):
        return self.a + self.b

e1 = E1(3,4)
assert e1.sum() == 7



from stdc import *

assert sizeof(Int8) == sizeof(UInt8) == 1
assert sizeof(Int16) == sizeof(UInt16) == 2
assert sizeof(Int32) == sizeof(UInt32) == 4
assert sizeof(Int64) == sizeof(UInt64) == 8

assert sizeof(Float) == 4
assert sizeof(Double) == 8

assert sizeof(Bool) == 1
assert sizeof(Pointer) in (4, 8)

x = Int32(42)
assert x.value == 42
x.value = 100
assert x.value == 100

Int32.read(addressof(x), 0) == 100
Int32.write(addressof(x), 0, 200)
assert x.value == 200

# test array
arr = Int32.array(3)
arr[0] = 10
arr[1] = 20
arr[2] = 30
assert arr[0] == 10
assert arr[1] == 20
assert arr[2] == 30

# test malloc, memset, memcpy
p = malloc(3 * sizeof(Int32))
memset(p, 0, 3 * sizeof(Int32))
memcpy(p, addressof(arr), 3 * sizeof(Int32))
for i in range(3):
    assert arr[i] == Int32.read(p, i)

assert memcmp(p, addressof(arr), 3 * sizeof(Int32)) == 0

# test free
free(p)

# test float
y = Double.array(3)
y[0] = 1.1
y[1] = 2.2
y[2] = 3.3
assert Double.read(addressof(y), 0) == 1.1
assert Double.read(addressof(y), 1) == 2.2
assert Double.read(addressof(y), 2) == 3.3

# test read_cstr and write_cstr
a = Char.array(20)
write_cstr(addressof(a), "hello")
assert read_cstr(addressof(a)) == "hello"

a[3] = 0
assert read_cstr(addressof(a)) == "hel"

# test read_bytes and write_bytes
assert read_bytes(addressof(a), 5) == b'hel\x00o'



import random

a = [random.randint(-100000, 100000) for i in range(100000)]

def __qsort(a: list, L: int, R: int):
    if L >= R: return;
    mid = a[(R+L)//2];
    i, j = L, R
    while i<=j:
        while a[i]<mid: i+=1
        while a[j]>mid: j-=1
        if i<=j:
            a[i], a[j] = a[j], a[i]
            i+=1
            j-=1
    __qsort(a, L, j)
    __qsort(a, i, R)

from dis import dis
# dis(__qsort)

__qsort(a, 0, len(a)-1)


from typing import TypeVar, Iterable

def Counter[T](iterable: Iterable[T]):
    a: dict[T, int] = {}
    for x in iterable:
        if x in a:
            a[x] += 1
        else:
            a[x] = 1
    return a


class defaultdict(dict):
    def __init__(self, default_factory, *args):
        super().__init__(*args)
        self.default_factory = default_factory

    def __missing__(self, key):
        self[key] = self.default_factory()
        return self[key]

    def __repr__(self) -> str:
        return f"defaultdict({self.default_factory}, {super().__repr__()})"

    def copy(self):
        return defaultdict(self.default_factory, self)


class deque[T]:
    _head: int
    _tail: int
    _maxlen: int | None
    _capacity: int
    _data: list[T]

    def __init__(self, iterable: Iterable[T] = None, maxlen: int | None = None):
        if maxlen is not None:
            assert maxlen > 0

        self._head = 0
        self._tail = 0
        self._maxlen = maxlen
        self._capacity = 8 if maxlen is None else maxlen + 1
        self._data = [None] * self._capacity # type: ignore

        if iterable is not None:
            self.extend(iterable)

    @property
    def maxlen(self) -> int | None:
        return self._maxlen

    def __resize_2x(self):
        backup = list(self)
        self._capacity *= 2
        self._head = 0
        self._tail = len(backup)
        self._data.clear()
        self._data.extend(backup)
        self._data.extend([None] * (self._capacity - len(backup)))

    def append(self, x: T):
        if (self._tail + 1) % self._capacity == self._head:
            if self._maxlen is None:
                self.__resize_2x()
            else:
                self.popleft()
        self._data[self._tail] = x
        self._tail = (self._tail + 1) % self._capacity

    def appendleft(self, x: T):
        if (self._tail + 1) % self._capacity == self._head:
            if self._maxlen is None:
                self.__resize_2x()
            else:
                self.pop()
        self._head = (self._head - 1) % self._capacity
        self._data[self._head] = x

    def copy(self):
        return deque(self, maxlen=self.maxlen)
    
    def count(self, x: T) -> int:
        n = 0
        for item in self:
            if item == x:
                n += 1
        return n
    
    def extend(self, iterable: Iterable[T]):
        for x in iterable:
            self.append(x)

    def extendleft(self, iterable: Iterable[T]):
        for x in iterable:
            self.appendleft(x)
    
    def pop(self) -> T:
        if self._head == self._tail:
            raise IndexError("pop from an empty deque")
        self._tail = (self._tail - 1) % self._capacity
        x = self._data[self._tail]
        self._data[self._tail] = None
        return x
    
    def popleft(self) -> T:
        if self._head == self._tail:
            raise IndexError("pop from an empty deque")
        x = self._data[self._head]
        self._data[self._head] = None
        self._head = (self._head + 1) % self._capacity
        return x
    
    def clear(self):
        i = self._head
        while i != self._tail:
            self._data[i] = None # type: ignore
            i = (i + 1) % self._capacity
        self._head = 0
        self._tail = 0

    def rotate(self, n: int = 1):
        if len(self) == 0:
            return
        if n > 0:
            n = n % len(self)
            for _ in range(n):
                self.appendleft(self.pop())
        elif n < 0:
            n = -n % len(self)
            for _ in range(n):
                self.append(self.popleft())

    def __len__(self) -> int:
        return (self._tail - self._head) % self._capacity

    def __contains__(self, x: object) -> bool:
        for item in self:
            if item == x:
                return True
        return False
    
    def __iter__(self):
        i = self._head
        while i != self._tail:
            yield self._data[i]
            i = (i + 1) % self._capacity

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, deque):
            return NotImplemented
        if len(self) != len(other):
            return False
        for x, y in zip(self, other):
            if x != y:
                return False
        return True
    
    def __ne__(self, other: object) -> bool:
        if not isinstance(other, deque):
            return NotImplemented
        return not self == other
    
    def __repr__(self) -> str:
        if self.maxlen is None:
            return f"deque({list(self)!r})"
        return f"deque({list(self)!r}, maxlen={self.maxlen})"




def compress(data: bytes) -> bytes:
    """Compress the given data into LZ4 block format.
    
    This function is equivalent to `lz4.block.compress` of https://pypi.org/project/lz4/.
    """

def decompress(data: bytes) -> bytes:
    """Decompress the given LZ4 block format data produced by `lz4.compress()`.
    
    This function is equivalent to `lz4.block.decompress` of https://pypi.org/project/lz4/.
    """



def f(a, b, *args):
    return a + b + sum(args)

assert f(1, 2, 3, 4) == 10

a = [5, 6, 7, 8]
assert f(*a) == 26

def g(*args):
    return f(*args)

assert g(1, 2, 3, 4) == 10
assert g(*a) == 26

def f(a, b, *args, c=16):
    return a + b + sum(args) + c

assert f(1, 2, 3, 4) == 26
assert f(1, 2, 3, 4, c=32) == 42

assert f(*a, c=-26) == 0


a, b = 1, 2
a, b = b, a
assert a == 2
assert b == 1

a, *b = 1, 2, 3, 4
assert a == 1
assert b == [2, 3, 4]

a, *b = [1]
assert a == 1
assert b == []

# test perfect forwarding
def f0(a, b, *args):
    return a + b + sum(args)

x = f0(1, 2, 3, 4)
assert x == 10

a = [1, 2, 3, 4]
x = f0(*a)
assert x == 10

def f1(a):
    return a

try:
    x = f1(*[1, 2, 3, 4])
    exit(1)
except TypeError:
    pass


def g(*args, **kwargs):
    return args, kwargs

def f(a, b, *args, c=1, **kwargs):
    return g(a, b, *args, c=c, **kwargs)

args, kwargs = f(1, 2, 3, 4, c=5, d=6, e=-6.0)
assert args == (1, 2, 3, 4)
assert kwargs == {'c': 5, 'd': 6, 'e': -6.0}


C_INT_TYPES = [
    'char', 'short', 'int', 'long', 'long long',
    'signed char', 'signed short', 'signed int', 'signed long', 'signed long long',
    'unsigned char', 'unsigned short', 'unsigned int', 'unsigned long', 'unsigned long long',
    'int8_t', 'int16_t', 'int32_t', 'int64_t',
    'uint8_t', 'uint16_t', 'uint32_t', 'uint64_t',
    'intptr_t', 'uintptr_t',
    'ptrdiff_t', 'size_t',
    'unsigned', 'signed',
    'py_i64',
]

C_FLOAT_TYPES = [
    'float', 'double',
    'py_f64',
]

C_BOOL_TYPES = [
    'bool', '_Bool',
]

C_STRING_TYPES = [
    'const char*',
    'const char *',
]

VMATH_TYPES = [
    'vec2', 'vec3', 'vec2i', 'vec3i',
    'mat3x3', 'color32'
]



assert eval('1+1') == 2
assert eval('[1,2,3]') == [1,2,3]

def f(x):
    return eval('x')

assert f(1) == 1


a = 0
assert eval('a') == 0

exec('a = 1')
assert a == 1

def f(a):
    exec('a = 3')
    return a

assert f(2) == 3

exec(
    "exec('a = eval(\"3 + 5\")')"
)
assert a == 8

def f():
    b = 1
    exec(
        "exec('b = eval(\"3 + 5\")')"
    )
    assert b == 8

def abc():
    g = {}
    exec('a=1', g)
    return g['a']

res = abc()
assert (res==1), res


# test locals and globals
assert eval('a', {'a': 2}) == 2

globals = {'a': 1}
locals = {'a': 1}

exec('a=2', globals, locals)
assert locals == {'a': 2}
assert globals == {'a': 1}

exec('a=2', globals)
assert globals == {'a': 2}

globals = {'a': 2}
locals = {'b': 3}
assert eval('a*b', globals, locals) == 6

code = compile('a*b', '<string>', 'eval')
assert eval(code, globals, locals) == 6

try:
    exec('a*b*c', globals, locals)
    exit(1)
except NameError:
    pass

# https://github.com/pocketpy/pocketpy/issues/339
res = []
code = '\nres.append(x)\ndef f():\n  res.append(x)\nf()\n'
x = 33
exec(code, {'x': 42, 'res': res})
assert res == [42, 42]
assert x == 33

# test removing trailing newlines
assert eval('[1, 2, 3]\n  \n') == [1, 2, 3]


UPPER_BOUND = 5000000
PREFIX = 32338

# exit(0)

class Node:
    def __init__(self):
        self.children = {}
        self.terminal = False


class Sieve:
    def __init__(self, limit):
        self.limit = limit
        self.prime = [False] * (limit + 1)

    def to_list(self):
        result = [2, 3]
        for p in range(5, self.limit + 1):
            if self.prime[p]:
                result.append(p)
        return result

    def omit_squares(self):
        r = 5
        while r * r < self.limit:
            if self.prime[r]:
                i = r * r
                while i < self.limit:
                    self.prime[i] = False
                    i = i + r * r
            r += 1
        return self

    def step1(self, x, y):
        n = (4 * x * x) + (y * y)
        if n <= self.limit and (n % 12 == 1 or n % 12 == 5):
            self.prime[n] = not self.prime[n]

    def step2(self, x, y):
        n = (3 * x * x) + (y * y)
        if n <= self.limit and n % 12 == 7:
            self.prime[n] = not self.prime[n]

    def step3(self, x, y):
        n = (3 * x * x) - (y * y)
        if x > y and n <= self.limit and n % 12 == 11:
            self.prime[n] = not self.prime[n]

    def loop_y(self, x):
        y = 1
        while y * y < self.limit:
            self.step1(x, y)
            self.step2(x, y)
            self.step3(x, y)
            y += 1

    def loop_x(self):
        x = 1
        while x * x < self.limit:
            self.loop_y(x)
            x += 1

    def calc(self):
        self.loop_x()
        return self.omit_squares()


def generate_trie(l):
    root = Node()
    for el in l:
        head = root
        for ch in str(el):
            if ch not in head.children:
                head.children[ch] = Node()
            head = head.children[ch]
        head.terminal = True
    return root


def find(upper_bound, prefix):
    primes = Sieve(upper_bound).calc()
    str_prefix = str(prefix)
    head = generate_trie(primes.to_list())
    for ch in str_prefix:
        head = head.children.get(ch)
        if head is None:    # either ch does not exist or the value is None
            return None

    queue, result = [(head, str_prefix)], []
    while queue:
        top, prefix = queue.pop()
        if top.terminal:
            result.append(int(prefix))
        for ch, v in top.children.items():
            queue.insert(0, (v, prefix + ch))

    result.sort()
    return result


def verify():
    left = [2, 23, 29]
    right = find(100, 2)
    if left != right:
        print(f"{left} != {right}")
        exit(1)

verify()
results = find(UPPER_BOUND, PREFIX)
assert results == [323381, 323383, 3233803, 3233809, 3233851, 3233863, 3233873, 3233887, 3233897]



from functools import cache

class A:
    def __init__(self, x):
        self._x = x

    @property
    def x(self):
        return self._x
    
    def __call__(self, b):
        return self.x + b
    
a = A(1)
assert a.x == 1
assert a(2) == 3

class B:
    def __init__(self):
        self._x = 1

    def _x_setter(self, v):
        self._x = v

B.x = property(
        lambda self: self._x,
        B._x_setter
    )

b = B()
assert b.x == 1
b.x = 2
assert b.x == 2

@cache
def fib(n):
    # print(f'fib({n})')
    if n < 2:
        return n
    return fib(n-1) + fib(n-2)

assert fib(32) == 2178309

def wrapped(cls):
    return int

@wrapped
@wrapped
@wrapped
@wrapped
class A:
    def __init__(self) -> None:
        pass

assert A('123') == 123

# validate the decorator order
res = []

def w(x):
    res.append('w')
    return x

def w1(x):
    res.append('w1')
    return x

def w2(x):
    res.append('w2')
    return x

@w
@w1
@w2
def f():
    pass

f()
assert res == ['w2', 'w1', 'w']



from bisect import bisect_left, bisect_right, insort_left, insort_right

a = [1, 1, 2, 6, 7, 8, 16, 22]

assert bisect_left(a, 23) == 8  # 23 does not exist in the list
assert bisect_left(a, 22) == 7  # 22 exists in the list
assert bisect_left(a, 0) == 0   # 0 does not exist in the list
assert bisect_left(a, 1) == 0   # 1 exists in the list
assert bisect_left(a, 5) == 3   # 5 does not exist in the list
assert bisect_left(a, 6) == 3   # 6 does exist in the list

# test bisect_right
assert bisect_right(a, 23) == 8  # 23 does not exist in the list
assert bisect_right(a, 22) == 8  # 22 exists in the list
assert bisect_right(a, 0) == 0   # 0 does not exist in the list
assert bisect_right(a, 1) == 2   # 1 exists in the list
assert bisect_right(a, 5) == 3   # 5 does not exist in the list
assert bisect_right(a, 6) == 4   # 6 does exist in the list

# test insort_left
insort_left(a, 23)
assert a == [1, 1, 2, 6, 7, 8, 16, 22, 23]

insort_left(a, 0)
assert a == [0, 1, 1, 2, 6, 7, 8, 16, 22, 23]

insort_left(a, 5)
assert a == [0, 1, 1, 2, 5, 6, 7, 8, 16, 22, 23]

insort_left(a, 1)
assert a == [0, 1, 1, 1, 2, 5, 6, 7, 8, 16, 22, 23]

# test insort_right
insort_right(a, 23)
assert a == [0, 1, 1, 1, 2, 5, 6, 7, 8, 16, 22, 23, 23]

insort_right(a, 0)
assert a == [0, 0, 1, 1, 1, 2, 5, 6, 7, 8, 16, 22, 23, 23]

insort_right(a, 5)
assert a == [0, 0, 1, 1, 1, 2, 5, 5, 6, 7, 8, 16, 22, 23, 23]

insort_right(a, 1)
assert a == [0, 0, 1, 1, 1, 1, 2, 5, 5, 6, 7, 8, 16, 22, 23, 23]


class A:
    def __init__(self, a, __b):
        self.a = a
        self.__b = __b

    @property
    def b(self):
        return self.__b

    def add(self):
        return self.a + self.b

    def sub(self):
        return self.a - self.b
    
a = A(1, 2)
assert a.add() == 3
assert a.sub() == -1

assert A.__base__ is object

class B(A):
    def __init__(self, a, b, c):
        super().__init__(a, b)
        self.c = c

    def add(self):
        return self.a + self.b + self.c

    def sub(self):
        return self.a - self.b - self.c

assert B.__base__ is A    

b = B(1, 2, 3)
assert b.add() == 6
assert b.sub() == -4

class C(B):
    def __init__(self, a, b, c, d):
        super().__init__(a, b, c)
        self.d = d

    def add(self):
        return self.a + self.b + self.c + self.d

    def sub(self):
        return self.a - self.b - self.c - self.d
    
assert C.__base__ is B

c = C(1, 2, 3, 4)
assert c.add() == 10
assert c.sub() == -8

class D(C):
    def __init__(self, a, b, c, d, e):
        super().__init__(a, b, c, d)
        self.e = e

    def add(self):
        return super().add() + self.e

    def sub(self):
        return super().sub() - self.e
    
assert D.__base__ is C

d = D(1, 2, 3, 4, 5)
assert d.add() == 15
assert d.sub() == -13

assert isinstance(1, int)
assert isinstance(1, object)
assert isinstance(C, type)
assert isinstance(C, object)
assert isinstance(d, object)
assert isinstance(d, C)
assert isinstance(d, B)
assert isinstance(d, A)
assert isinstance(object, object)
assert isinstance(type, object)

assert isinstance(1, (float, int))
assert isinstance(1, (float, object))
assert not isinstance(1, (float, str))
assert isinstance(object, (int, type, float))
assert not isinstance(object, (int, float, str))

try:
    isinstance(1, (1, 2))
    exit(1)
except TypeError:
    pass

try:
    isinstance(1, 1)
    exit(1)
except TypeError:
    pass

class A:
    a = 1
    b = 2

assert A.a == 1
assert A.b == 2

class B(A):
    b = 3
    c = 4

# assert B.a == 1  ...bug here
assert B.b == 3
assert B.c == 4

if 1:
    class TrueClass:
        pass
else:
    class FalseClass:
        pass

assert TrueClass

# staticmethod and classmethod
class A():
    dd = 2

    def __init__(self):
        self.a = 1
        
    @staticmethod
    def static_method(txt):
        return txt
    
    @classmethod
    def class_method(cls, txt):
        return cls.__name__ + txt

assert A.static_method(123) == 123
assert A.class_method('123') == 'A123'
assert A().static_method(123) == 123
assert A().class_method('123') == 'A123'
assert A.dd == 2
assert A().dd == 2

class B(A): pass

assert B.dd == 2
assert B().dd == 2

assert B.static_method(123) == 123
assert B.class_method('123') == 'B123'
assert B().static_method(123) == 123
assert B().class_method('123') == 'B123'

# test super() with classmethod
class BaseClass:
    @classmethod
    def f(cls):
        return 'BaseClass'

class DerivedClass(BaseClass):
    @classmethod
    def f(cls):
        return super().f()

    
assert DerivedClass.f() == 'BaseClass'



class Test[T]:
    def __init__(self, value: T):
        self.value = value

    def get_value(self) -> T:
        return self.value


def add[T: int|str|float](a: T, b: T) -> T:
    return a + b # type: ignore

res = add(1, 2)
assert res == 3

test = Test(1)
assert test.get_value() == 1

# test multiple
class Test2[T: int, U]: pass
class Test3[T: int | str, U: float, R: list]: pass



# test deletion
rnd = 0
keys = []
while True:
    keys.append(rnd)
    rnd = ((rnd * 5) + 1) & 1023
    if rnd == 0:
        break

assert len(keys) == 1024

a = {k: k for k in keys}

for i in range(10000):
    if i % 2 == 0:
        # del all keys
        for k in keys:
            del a[k]
        assert len(a) == 0
    else:
        # add keys back
        for k in keys:
            a[k] = k
        assert len(a) == len(keys)

assert len(a) == len(keys)
assert list(a.keys()) == keys   # order matters



from vmath import mat3x3, vec2, vec3, vec2i, vec3i
import random
import math

a = vec2(1.5, 2)
assert a.x == 1.5
assert a.y == 2

# 出于对精度转换的考虑,在本测试中具体将采用str(floating_num)[:6]来比较两个浮点数是否相等

# test vec2--------------------------------------------------------------------

def rotated_vec2(vec_2: vec2, radians: float):
    cos_theta = math.cos(radians)
    sin_theta = math.sin(radians)
    new_x = vec_2.x * cos_theta - vec_2.y * sin_theta
    new_y = vec_2.x * sin_theta + vec_2.y * cos_theta
    return vec2(new_x, new_y)

# 生成随机测试目标
min_num = -10.0
max_num = 10.0
test_vec2 = vec2(*tuple([random.uniform(min_num, max_num) for _ in range(2)]))
test_vec2_2 = vec2(*tuple([random.uniform(min_num, max_num) for _ in range(2)]))
static_test_vec2_float = vec2(3.18, -1.09)
static_test_vec2_int = vec2(278, -1391)

# test __repr__
assert str(static_test_vec2_float).startswith('vec2(')
assert str(static_test_vec2_int).startswith('vec2(')

# test rotate
test_vec2_copy = test_vec2
radians = random.uniform(-10*math.pi, 10*math.pi)
test_vec2_copy = rotated_vec2(test_vec2_copy, radians)
res = test_vec2.rotate(radians)
assert (res == test_vec2_copy), (res, test_vec2_copy, test_vec2)

# test smooth_damp
vel = vec2(0, 0)
ret, vel = vec2.smooth_damp(vec2(1, 2), vec2(3, 4), vel, 0.2, 0.001, 0.05)
assert isinstance(ret, vec2)
assert vel.length() > 0

# test vec3--------------------------------------------------------------------
# 生成随机测试目标
min_num = -10.0
max_num = 10.0
test_vec3 = vec3(*tuple([random.uniform(min_num, max_num) for _ in range(3)]))
static_test_vec3_float = vec3(3.1886954323, -1098399.59932453432, 9.00000000000002765)
static_test_vec3_int = vec3(278, -13919730938747, 1364223456756456)

# test __repr__
assert str(static_test_vec3_float).startswith('vec3(')
assert str(static_test_vec3_int).startswith('vec3(')

# test copy
element_name_list = ['x', 'y', 'z']
element_value_list = [getattr(test_vec3, attr) for attr in element_name_list]
copy_element_value_list = [getattr(test_vec3, attr) for attr in element_name_list]
assert element_value_list == copy_element_value_list

# test mat3x3--------------------------------------------------------------------
def mat_to_str_list(mat):
    ret = [[0,0,0], [0,0,0], [0,0,0]]
    for i in range(3):
        for j in range(3):
            ret[i][j] = str(round(mat[i, j], 2))[:6]
    return ret

def mat_list_to_str_list(mat_list):
    ret = [[0,0,0], [0,0,0], [0,0,0]]
    for i in range(3):
        for j in range(3):
            ret[i][j] = str(round(mat_list[i][j], 2))[:6]
    return ret

def mat_to_list(mat):
    ret = [[0,0,0], [0,0,0], [0,0,0]]
    for i in range(3):
        for j in range(3):
            ret[i][j] = mat[i, j]
    return ret
    
def mat_round(mat, pos):
    '''
    对mat的副本的每一个元素执行round(element, pos)，返回副本
    用于校对元素是浮点数的矩阵
    '''
    ret = mat.copy()
    
    for i, row in enumerate(ret):
        for j, element in enumerate(row):
            row[j] = round(element, pos)
        ret[i] = row
    
    return ret

def get_row(mat, row_index):
    '''
    返回mat的row_index行元素构成的列表
    '''
    ret = []
    for i in range(3):
        ret.append(mat[row_index, i])
    return ret

def get_col(mat, col_index):
    '''
    返回mat的col_index列元素构成的列表
    '''
    ret = []
    for i in range(3):
        ret.append(mat[i, col_index])
    return ret

def calculate_inverse(matrix):
    '''
    返回逆矩阵
    '''
    # 获取矩阵的行数和列数
    rows = len(matrix)
    cols = len(matrix[0])
    # 确保矩阵是方阵
    if rows != cols:
        raise ValueError("输入矩阵必须是方阵")
    # 构建单位矩阵
    identity = [[1 if i == j else 0 for j in range(cols)] for i in range(rows)]
    # 将单位矩阵与输入矩阵进行初等行变换
    augmented_matrix = [row + identity[i] for i, row in enumerate(matrix)]
    # 初等行变换，将输入矩阵转化为单位矩阵，同时在另一边进行相同的行变换
    for i in range(cols):
        pivot = augmented_matrix[i][i]
        if pivot == 0:
            raise ValueError("输入矩阵不可逆")
        scale_row(augmented_matrix, i, 1/pivot)
        for j in range(cols):
            if j != i:
                scale = augmented_matrix[j][i]
                row_operation(augmented_matrix, j, i, -scale)
    # 提取逆矩阵
    inverse_matrix = [row[cols:] for row in augmented_matrix]
    return inverse_matrix

def scale_row(matrix, row, scale):
    matrix[row] = [element * scale for element in matrix[row]]

def row_operation(matrix, target_row, source_row, scale):
    matrix[target_row] = [target_element + scale * source_element for target_element, source_element in zip(matrix[target_row], matrix[source_row])]
# 生成随机测试目标
min_num = -10.0
max_num = 10.0
test_mat = mat3x3(*[random.uniform(min_num, max_num) for _ in range(9)])
static_test_mat_float= mat3x3(
    7.264189733952545, -5.432187523625671, 1.8765304152872613,
    -2.4910524352374734, 8.989660807513068, -0.7168824333280513,
    9.558042327611506, -3.336280256662496, 4.951381528057387
)

static_test_mat_float_inv = mat3x3( 0.32265243,  0.15808159, -0.09939472,
        0.04199553,  0.13813096,  0.00408326,
       -0.59454451, -0.21208362,  0.39658464)

static_test_mat_int = mat3x3(1, 2, 3, 4, 5, 6, 7, 8, 9)

# test incorrect number of parameters is passed
for i in range(20):
    
    if i in [0, 9]:
        continue
    
    try:
        test_mat_copy = mat3x3(*tuple([e+0.1 for e in range(i)]))
        
        # 既然参数数量不是合法的0个或9个,并且这里也没有触发TypeError,那么引发测试失败
        print(f'When there are {i} arguments, no TypeError is triggered')
        exit(1)
        
    except TypeError:
        pass

        
# test copy
test_mat_copy = test_mat.copy()
assert test_mat is not test_mat_copy
assert test_mat == test_mat_copy

try:
    test_mat[1,2,3]
except IndexError:
    pass

try:
    test_mat[-1, 4]
    raise Exception('未能触发错误拦截, 此处应当报错 IndexError("index out of range")')
except IndexError:
    pass

# test __setitem__ and __getitem__
test_mat_copy = test_mat.copy()
test_mat_copy[1, 2] = 1
assert test_mat_copy[1, 2] == 1

try:
    test_mat[1,2,3] = 1
    raise Exception('未能触发错误拦截, 此处应当报错 TypeError("Mat3x3.__setitem__ takes a tuple of 2 integers")')
except IndexError:
    pass

try:
    test_mat[-1, 4] = 1
    raise Exception('未能触发错误拦截, 此处应当报错 IndexError("index out of range")')
except IndexError:
    pass

# test matmul
test_mat_copy = test_mat.copy()
test_mat_copy_2 = test_mat.copy()
result_mat = test_mat_copy @ test_mat_copy_2
correct_result_mat = mat3x3.zeros()
for i in range(3):
    for j in range(3):
        correct_result_mat[i, j] = sum([e1*e2 for e1, e2 in zip(get_row(test_mat_copy, i), get_col(test_mat_copy_2, j))])
assert result_mat == correct_result_mat

# test determinant
test_mat_copy = test_mat.copy()
test_mat_copy.determinant()

# test __repr__
assert str(static_test_mat_float)
assert str(static_test_mat_int)


# 此处测试不完全, 未验证正确性
# test interface of "@" "matmul" "__matmul__" with vec3 and error handling
test_mat_copy = test_mat.copy()
test_mat_copy @ vec3(83,-9.12, 0.2983)
try:
    test_mat_copy @ 12345
    exit(1)
except TypeError:
    pass


# test inverse
assert ~static_test_mat_float == static_test_mat_float_inv == static_test_mat_float.inverse()
assert static_test_mat_float.inverse_() is None
assert static_test_mat_float == static_test_mat_float_inv

try:
    ~mat3x3(*[1, 2, 3, 2, 4, 6, 3, 6, 9])
    raise Exception('未能拦截错误 ValueError("matrix is not invertible") 在 test_mat_copy 的行列式为0')
except ZeroDivisionError:
    pass

# test zeros
assert mat3x3(*[0 for _ in range(9)]) == mat3x3.zeros()

# test identity
assert mat3x3(*[1,0,0,0,1,0,0,0,1]) == mat3x3.identity()


# test affine transformations-----------------------------------------------
# test trs
def trs(t, radian, s):
    cr = math.cos(radian)
    sr = math.sin(radian)
    elements = [[s[0] * cr, -s[1] * sr, t[0]],
                [s[0] * sr, s[1] * cr, t[1]],
                [0.0, 0.0, 1.0]]
    return elements

test_vec2_copy = test_vec2
test_vec2_2_copy = test_vec2_2

test_vec2_list = [test_vec2_copy.x, test_vec2_copy.y]
test_vec2_2_list = [test_vec2_2_copy.x, test_vec2_2_copy.y]

radian = random.uniform(-10*math.pi, 10*math.pi)

mat3x3.trs(test_vec2_copy, radian, test_vec2_2_copy)

a = mat3x3.zeros()
a.copy_trs_(test_vec2_copy, radian, test_vec2_2_copy)
assert a == mat3x3.trs(test_vec2_copy, radian, test_vec2_2_copy)

# test translation
test_mat_copy = test_mat.copy()
assert test_mat_copy.t() == vec2(test_mat_copy[0, 2], test_mat_copy[1, 2])

# 该方法的测试未验证计算的准确性
# test rotation
test_mat_copy = test_mat.copy()
assert type(test_mat_copy.r()) is float


# test scale
test_mat_copy = test_mat.copy()
temp_vec2 = test_mat_copy.s()

# test transform_point
test_mat_copy = test_mat.copy()
test_mat_copy = test_mat.copy()
test_vec2_copy = test_vec2
temp_vec2 = test_mat_copy.transform_point(test_vec2_copy)

# test transform_vector
test_mat_copy = test_mat.copy()
test_mat_copy = test_mat.copy()
test_vec2_copy = test_vec2
temp_vec2 = test_mat_copy.transform_vector(test_vec2_copy)

val = vec2.angle(vec2(-1, 0), vec2(0, -1))
assert 1.57 < val < 1.58

# test about staticmethod
d = mat3x3.identity()
d1 = mat3x3(1, 2, 3, 4, 5, 6, 7, 8, 9).identity()
assert d == d1
assert d.copy_(mat3x3.zeros()) is None
assert d == mat3x3.zeros()

d = mat3x3.identity()
assert d @ mat3x3.zeros() == mat3x3.zeros()
assert d == mat3x3.identity()
assert d.matmul(mat3x3.zeros(), d) is None
assert d == mat3x3.zeros()

try:
    assert d[6, 6]
    exit(1)
except IndexError:
    pass

# test vec * vec
assert vec2(1, 2) * vec2(3, 4) == vec2(3, 8)

# test vec2i and vec3i

a = vec2i(1, 2)
assert a.x == 1
assert a.y == 2

assert a == vec2i(1, 2)

a = vec3i(1, 2, 3)
assert a.x == 1
assert a.y == 2
assert a.z == 3

assert a == vec3i(1, 2, 3)
assert a.with_x(2) == vec3i(2, 2, 3)
assert a.with_y(3) == vec3i(1, 3, 3)
assert a.with_z(4) == vec3i(1, 2, 4)

# test vec2.with_z
assert vec2(1, 2).with_z(3) == vec3(1, 2, 3)
# test vec3.xy
assert vec3(1, 2, 3).xy == vec2(1, 2)
# test vec3.ONE
assert vec3.ONE == vec3(1, 1, 1)
# test vec3.ZERO
assert vec3.ZERO == vec3(0, 0, 0)
# test vec3.with_xy
assert vec3(1, 2, 3).with_xy(vec2(4, 5)) == vec3(4, 5, 3)

# test vec2i and vec3i
assert vec2i.ONE == vec2i(1, 1)
assert vec2i.ZERO == vec2i(0, 0)
assert vec2i.LEFT == vec2i(-1, 0)
assert vec2i.RIGHT == vec2i(1, 0)
assert vec2i.UP == vec2i(0, -1)
assert vec2i.DOWN == vec2i(0, 1)

assert vec3i.ONE == vec3i(1, 1, 1)
assert vec3i.ZERO == vec3i(0, 0, 0)

assert vec2i(1, 2) + vec2i(3, 4) == vec2i(4, 6)
assert vec2i(1, 2) - vec2i(3, 4) == vec2i(-2, -2)
assert vec2i(1, 2) * vec2i(3, 4) == vec2i(3, 8)
assert vec2i(1, 2) * 2 == vec2i(2, 4)
assert vec2i(1, 2).dot(vec2i(3, 4)) == 11

assert vec3i(1, 2, 3) + vec3i(4, 5, 6) == vec3i(5, 7, 9)
assert vec3i(1, 2, 3) - vec3i(4, 5, 6) == vec3i(-3, -3, -3)
assert vec3i(1, 2, 3) * vec3i(4, 5, 6) == vec3i(4, 10, 18)
assert vec3i(1, 2, 3) * 2 == vec3i(2, 4, 6)
assert vec3i(1, 2, 3).dot(vec3i(4, 5, 6)) == 32

assert vec2i(3, 5) // 2 == vec2i(1, 2)
assert vec3i(3, 5, 8) // 2 == vec3i(1, 2, 4)

a = {}
a[vec2i(1, 2)] = 1
assert a[vec2i(1, 2)] == 1
a[vec3i(1, 2, 3)] = 2
assert a[vec3i(1, 2, 3)] == 2

assert vec2(vec2i.LEFT) == vec2(-1, 0)
assert vec2(vec2i.RIGHT) == vec2(1, 0)
assert vec3(vec3i.ONE) == vec3(1, 1, 1)
assert vec3(vec3i.ZERO) == vec3(0, 0, 0)

x, y = vec2i(1, 2)
assert x == 1 and y == 2
x, y, z = vec3i(1, 2, 3)
assert x == 1 and y == 2 and z == 3
x, y = vec2(3.0, 4.0)
assert x == 3.0 and y == 4.0
x, y, z = vec3(1.0, 2.0, 3.0)
assert x == 1.0 and y == 2.0 and z == 3.0

assert hash(vec2i(11, -1)) == 51539607551
assert hash(vec3i(11, -1, 0)) == 13194139467776

d = {vec2i(12, 12): 1035.313708305359, vec2i(12, 11): 2059.313708305359, vec2i(12, 13): 2059.313708305359, vec2i(11, 12): 2059.313708305359, vec2i(13, 12): 2059.313708305359, vec2i(13, 11): 3083.313708305359, vec2i(13, 13): 3083.313708305359, vec2i(14, 12): 3083.313708305359, vec2i(12, 14): 3083.313708305359, vec2i(11, 13): 3083.313708305359, vec2i(12, 10): 3083.313708305359, vec2i(11, 11): 3083.313708305359, vec2i(10, 12): 3083.313708305359, vec2i(13, 14): 4107.313708305359, vec2i(14, 13): 4107.313708305359, vec2i(14, 11): 4107.313708305359, vec2i(15, 12): 4107.313708305359, vec2i(12, 15): 4107.313708305359, vec2i(11, 14): 4107.313708305359, vec2i(13, 10): 4107.313708305359, vec2i(10, 13): 4107.313708305359, vec2i(11, 10): 4107.313708305359, vec2i(10, 11): 4107.313708305359, vec2i(12, 9): 4107.313708305359, vec2i(9, 12): 4107.313708305359, vec2i(14, 14): 5131.313708305359, vec2i(15, 13): 5131.313708305359, vec2i(13, 15): 5131.313708305359, vec2i(15, 11): 5131.313708305359, vec2i(16, 12): 5131.313708305359, vec2i(12, 16): 5131.313708305359, vec2i(11, 15): 5131.313708305359, vec2i(14, 10): 5131.313708305359, vec2i(10, 14): 5131.313708305359, vec2i(13, 9): 5131.313708305359, vec2i(9, 13): 5131.313708305359}
d[vec2i(11, 9)] = 1

e = {}
for i in range(-1000, 10000):
    e[vec2i(12, i)] = i
    e[vec2i(11, i)] = i
    e[vec2i(13, i)] = i
    e[vec2i(i, 12)] = i
    e[vec2i(i, 11)] = i
    e[vec2i(i, 13)] = i



from collections import Counter, deque, defaultdict
import random
import pickle
import gc

# test defaultdict
assert issubclass(defaultdict, dict)
a = defaultdict(int)
a['1'] += 1
assert a == {'1': 1}
a = defaultdict(list)
a['1'].append(1)
assert a == {'1': [1]}

q = deque()
q.append(1)
q.append(2)
q.appendleft(3)
q.append(4)
assert len(q) == 4
assert q == deque([3, 1, 2, 4])
assert q.popleft() == 3
assert q.pop() == 4
assert len(q) == 2
assert q == deque([1, 2])

# ADDING TESTS FROM CPYTHON's test_deque.py file

############ TEST basics###############


def assertEqual(a, b):
    if a == b:
        return
    print(a)
    print(b)
    raise AssertionError

def assertNotEqual(a, b):
    if a != b:
        return
    print(a)
    print(b)
    raise AssertionError

def printFailed(function_name, *args, **kwargs):
    print("X Failed Tests for {} for args: {} {}".format(str(function_name), str(args), str(kwargs)))


BIG = 10000


def fail():
    raise SyntaxError
    yield 1

d = deque()
assertEqual(len(d), 0)
assertEqual(list(d), [])

d = deque(range(6))
assertEqual(list(d), list(range(6)))
d = deque(range(7)) # [0, 1, 2, 3, 4, 5, 6]
# print(d._data, d._head, d._tail, d._capacity)
assertEqual(list(d), list(range(7)))
d = deque(range(8))
assertEqual(list(d), list(range(8)))
d = deque(range(9))
assertEqual(list(d), list(range(9)))

d = deque(range(200))
for i in range(200, 400):
    d.append(i)

assertEqual(len(d), 400)
assertEqual(list(d), list(range(400)))

for i in reversed(range(-200, 0)):
    d.appendleft(i)

assertEqual(len(d), 600)
assertEqual(list(d), list(range(-200, 400)))

left = [d.popleft() for i in range(250)]
assertEqual(left, list(range(-200, 50)))
assertEqual(list(d), list(range(50, 400)))

right = [d.pop() for i in range(250)]
right.reverse()
assertEqual(right, list(range(150, 400)))
assertEqual(list(d), list(range(50, 150)))

######### TEST count()#################
for s in ('', 'abracadabra', 'simsalabim'*500+'abc'):
    s = list(s)
    d = deque(s)
    for letter in 'abcdefghijklmnopqrstuvwxyz':
        assertEqual(s.count(letter), d.count(letter))
try:
    d.count()
    printFailed("deque.count")
    exit(1)
except TypeError:
    pass

try:
    d.count(1, 2)
    printFailed("deque.count", 1, 2)
    exit(1)
except TypeError:
    pass

class ArithmeticError(Exception): pass

class BadCompare:
    def __eq__(self, other):
        raise ArithmeticError
    def __ne__(self, other):
        raise ArithmeticError


d = deque([1, 2, BadCompare(), 3])

try:
    d.count(2)
    printFailed("deque.count", 2)
    exit(1)
except ArithmeticError:
    pass

d = deque([1, 2, 3])
try:
    d.count(BadCompare())
    printFailed("deque.count", "BadCompare()")
    exit(1)
except ArithmeticError:
    pass


# class MutatingCompare:
#     def __eq__(self, other):
#         d.pop()
#         return True

# m = MutatingCompare()
# d = deque([1, 2, 3, m, 4, 5])
# m.d = d

# try:
#     d.count(3)
#     printFailed("deque.count", "MutatingCompare()")
#     exit(1)
# except RuntimeError:
#     pass

#### TEST comparisons == #####

d = deque('xabc')
d.popleft()
for e in [d, deque('abc'), deque('ab'), deque(), list(d)]:
    assertEqual(d == e, type(d) == type(e) and list(d) == list(e))
    assertEqual(d != e, not (type(d) == type(e) and list(d) == list(e)))

def get_args():
    args = map(deque, ('', 'a', 'b', 'ab', 'ba', 'abc', 'xba', 'xabc', 'cba'))
    return args

for x in get_args():
    for y in get_args():
        assertEqual(x == y, list(x) == list(y))
        assertEqual(x != y, list(x) != list(y))
        # assertEqual(x <  y, list(x) <  list(y))   # not currently supported
        # assertEqual(x <= y, list(x) <= list(y))   # not currently supported
        # assertEqual(x >  y, list(x) >  list(y))   # not currently supported
        # assertEqual(x >= y, list(x) >= list(y))   # not currently supported


############### TEST contains()#################

n = 200

d = deque(range(n))
for i in range(n):
    assertEqual(i in d, True)
assertEqual((n+1) not in d, True)


# class MutateCmp:
#     def __init__(self, deque, result):
#         self.deque = deque
#         self.result = result

#     def __eq__(self, other):
#         self.deque.clear()
#         return self.result


# # # Test detection of mutation during iteration
# d = deque(range(n))
# d[n//2] = MutateCmp(d, False)
# try:
#     n in d
#     printFailed("deque.__contains__", n)
#     exit(1)
# except RuntimeError:
#     pass


class BadCmp:
    def __eq__(self, other):
        raise RuntimeError
    def __ne__(self, other):
        raise RuntimeError


# # Test detection of comparison exceptions
d = deque(range(n))
d.append(BadCmp())
try:
    n in d
    printFailed("deque.__contains__", n)
    exit(1)
except RuntimeError:
    pass

######## TEST extend()################


d = deque('a')
try:
    d.extend(1)
    printFailed("deque.extend", 1)
    exit(1)
except TypeError:
    pass
d.extend('bcd')
assertEqual(list(d), list('abcd'))
d.extend(d.copy())
assertEqual(list(d), list('abcdabcd'))

###### TEST extend_left() ################

d = deque('a')
try:
    d.extendleft(1)
    printFailed("deque.extendleft", 1)
    exit(1)
except TypeError:
    pass
d.extendleft('bcd')
assertEqual(list(d), list(reversed('abcd')))
d.extendleft(d.copy())
assertEqual(list(d), list('abcddcba'))
d = deque()
d.extendleft(range(1000))
assertEqual(list(d), list(reversed(range(1000))))
try:
    d.extendleft(fail())
    printFailed("deque.extendleft", fail())
    exit(1)
except SyntaxError:
    pass


############ test rotate#############
s = tuple('abcde')
n = len(s)

d = deque(s)
d.rotate(1)             # verify rot(1)
assertEqual(''.join(d), 'eabcd')

d = deque(s)
d.rotate(-1)            # verify rot(-1)
assertEqual(''.join(d), 'bcdea')
d.rotate()              # check default to 1
assertEqual(tuple(d), s)

for i in range(n*3):
    d = deque(s)
    e = deque(d)
    # print(i, d, e)
    d.rotate(i)         # check vs. rot(1) n times
    for j in range(i):
        e.rotate(1)
    assertEqual(tuple(d), tuple(e))
    d.rotate(-i)        # check that it works in reverse
    assertEqual(tuple(d), s)
    e.rotate(n-i)       # check that it wraps forward
    assertEqual(tuple(e), s)

for i in range(n*3):
    d = deque(s)
    e = deque(d)
    d.rotate(-i)
    for j in range(i):
        e.rotate(-1)    # check vs. rot(-1) n times
    assertEqual(tuple(d), tuple(e))
    d.rotate(i)         # check that it works in reverse
    assertEqual(tuple(d), s)
    e.rotate(i-n)       # check that it wraps backaround
    assertEqual(tuple(e), s)

d = deque(s)
e = deque(s)
e.rotate(BIG+17)        # verify on long series of rotates
dr = d.rotate
for i in range(BIG+17):
    dr()
assertEqual(tuple(d), tuple(e))
try:
    d.rotate(1, 2)
    printFailed("deque.rotate", 1, 2)
    exit(1)
except TypeError:
    pass

try:
    d.rotate(1, 10)
    printFailed("deque.rotate", 1, 10)
    exit(1)
except TypeError:
    pass
d = deque()
d.rotate()              # rotate an empty deque
assertEqual(d, deque())


########## test len#############

d = deque('ab')
assertEqual(len(d), 2)
d.popleft()
assertEqual(len(d), 1)
d.pop()
assertEqual(len(d), 0)
try:
    d.pop()
    printFailed("deque.pop")
    exit(1)
except IndexError:
    pass
assertEqual(len(d), 0)
d.append('c')
assertEqual(len(d), 1)
d.appendleft('d')
assertEqual(len(d), 2)
d.clear()
assertEqual(len(d), 0)


############## test underflow#############
d = deque()
try:
    d.pop()
    printFailed("deque.pop")
    exit(1)
except IndexError:
    pass
try:
    d.popleft()
    printFailed("deque.popleft")
    exit(1)
except IndexError:
    pass

############## test clear#############
d = deque(range(100))
assertEqual(len(d), 100)
d.clear()
assertEqual(len(d), 0)
assertEqual(list(d), [])
d.clear()               # clear an empty deque
assertEqual(list(d), [])

# Handle comparison errors
d = deque(['a', 'b', BadCmp(), 'c'])
e = deque(d)

for x, y in zip(d, e):
    # verify that original order and values are retained.
    assertEqual(x is y, True)

# https://github.com/pocketpy/pocketpy/issues/447
names = ["Alice", "Bob", "Charlie"]
ages = [25, 30, 35]
cities = ["NY", "LA", "SF"]
result = []
for name, age, city in zip(names, ages, cities):
    result.append((name, age, city))
assertEqual(result, [("Alice", 25, "NY"), ("Bob", 30, "LA"), ("Charlie", 35, "SF")])

cities.pop()
result = []
for name, age, city in zip(names, ages, cities):
    result.append((name, age, city))
assertEqual(result, [("Alice", 25, "NY"), ("Bob", 30, "LA")])

########### test repr#############
d = deque(range(200))
e = eval(repr(d))
assertEqual(list(d), list(e))
d.append(None)
assertEqual(repr(d)[-19:], '7, 198, 199, None])')


######### test init #############

try:
    deque('abc', 2, 3)
    printFailed("deque", 'abc', 2, 3)
    exit(1)
except TypeError:
    pass
try:
    deque(1)
    printFailed("deque", 1)
    exit(1)
except TypeError:
    pass


######### test hash #############
try:
    hash(deque('abcd'))
except TypeError:
    pass


###### test long steady state queue pop left ########
for size in (0, 1, 2, 100, 1000):
    d = deque(range(size))
    append, pop = d.append, d.popleft
    for i in range(size, BIG):
        append(i)
        x = pop()
        if x != i - size:
            assertEqual(x, i-size)
    assertEqual(list(d), list(range(BIG-size, BIG)))


######## test long steady state queue pop right ########
for size in (0, 1, 2, 100, 1000):
    d = deque(reversed(range(size)))
    append, pop = d.appendleft, d.pop
    for i in range(size, BIG):
        append(i)
        x = pop()
        if x != i - size:
            assertEqual(x, i-size)
    assertEqual(list(reversed(list(d))),
                list(range(BIG-size, BIG)))

###### test big queue popleft ########
d = deque()
append, pop = d.append, d.popleft
for i in range(BIG):
    append(i)
for i in range(BIG):
    x = pop()
    if x != i:
        assertEqual(x, i)

###### test big queue pop right ########
d = deque()
append, pop = d.appendleft, d.pop
for i in range(BIG):
    append(i)
for i in range(BIG):
    x = pop()
    if x != i:
        assertEqual(x, i)


####### test big stack right########
d = deque()
append, pop = d.append, d.pop
for i in range(BIG):
    append(i)
for i in reversed(range(BIG)):
    x = pop()
    if x != i:
        assertEqual(x, i)
assertEqual(len(d), 0)


##### test big stack left ########
d = deque()
append, pop = d.appendleft, d.popleft
for i in range(BIG):
    append(i)
for i in reversed(range(BIG)):
    x = pop()
    if x != i:
        assertEqual(x, i)
assertEqual(len(d), 0)


##### test roundtrip iter init ########
d = deque(range(200))
e = deque(d)
assertNotEqual(id(d), id(e))
assertEqual(list(d), list(e))


########## test pickle #############

# d = deque(range(200))
# for _ in range(5 + 1):
#     s = pickle.dumps(d)
#     e = pickle.loads(s)
#     assertNotEqual(id(e), id(d))
#     assertEqual(list(e), list(d))

### test copy ########

mut = [10]
d = deque([mut])
e = d.copy()
assertEqual(list(d), list(e))
mut[0] = 11
assertNotEqual(id(d), id(e))
assertEqual(list(d), list(e))

### test reversed#$####

for s in ('abcd', range(2000)):
    assertEqual(list(reversed(deque(s))), list(reversed(s)))

d = deque()
for i in range(100):
    d.append(1)
    gc.collect()

# test maxlen
q = deque(maxlen=3)
assertEqual(q.maxlen, 3)
q.append(1)
q.append(2)
q.append(3)
assertEqual(list(q), [1, 2, 3])
assertEqual(q._data, [1, 2, 3, None])
q.append(4)
assertEqual(list(q), [2, 3, 4])
assertEqual(q._data, [None, 2, 3, 4])
q.appendleft(1)
assertEqual(list(q), [1, 2, 3])
assertEqual(q._data, [1, 2, 3, None])
q.appendleft(0)
assertEqual(list(q), [0, 1, 2])
assertEqual(q._data, [1, 2, None, 0])
q.pop()
assertEqual(list(q), [0, 1])
assertEqual(q._data, [1, None, None, 0])
assertEqual(len(q), 2)
assertEqual(q._capacity, 4)
q.popleft()
assertEqual(list(q), [1])
assertEqual(q._data, [1, None, None, None])





from typing import Callable, Literal, overload, Iterator, Self
from vmath import vec2i, color32

Neighborhood = Literal['Moore', 'von Neumann']

class array2d_like[T]:
    @property
    def n_cols(self) -> int: ...
    @property
    def n_rows(self) -> int: ...
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    @property
    def shape(self) -> vec2i: ...
    @property
    def numel(self) -> int: ...

    @overload
    def is_valid(self, col: int, row: int) -> bool: ...
    @overload
    def is_valid(self, pos: vec2i) -> bool: ...

    def get[R](self, col: int, row: int, default: R = None) -> T | R:
        """Get the value at the given position.
        
        If the position is out of bounds, returns the default value.
        """

    def index(self, value: T) -> vec2i:
        """Get the position of the first occurrence of the given value.
        
        Raises `ValueError` if the value is not found.
        """

    def render(self) -> str: ...
    def render_with_color(self, fg: array2d_like[color32 | None], bg: array2d_like[color32 | None]) -> str: ...

    def all(self: array2d_like[bool]) -> bool: ...
    def any(self: array2d_like[bool]) -> bool: ...

    def map[R](self, f: Callable[[T], R]) -> array2d[R]: ...
    def apply(self, f: Callable[[T], T]) -> None: ...
    def zip_with[R, U](self, other: array2d_like[U], f: Callable[[T, U], R]) -> array2d[R]: ...
    def copy(self) -> 'array2d[T]': ...
    def tolist(self) -> list[list[T]]: ...

    def __le__(self, other: T | array2d_like[T]) -> array2d[bool]: ...
    def __lt__(self, other: T | array2d_like[T]) -> array2d[bool]: ...
    def __ge__(self, other: T | array2d_like[T]) -> array2d[bool]: ...
    def __gt__(self, other: T | array2d_like[T]) -> array2d[bool]: ...
    def __eq__(self, other: T | array2d_like[T]) -> array2d[bool]: ...  # type: ignore
    def __ne__(self, other: T | array2d_like[T]) -> array2d[bool]: ...  # type: ignore

    def __add__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __sub__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __mul__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __truediv__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __floordiv__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __mod__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __pow__(self, other: T | array2d_like[T]) -> array2d[T]: ...

    def __and__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __or__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __xor__(self, other: T | array2d_like[T]) -> array2d[T]: ...
    def __invert__(self) -> array2d[T]: ...

    def __iter__(self) -> Iterator[tuple[vec2i, T]]: ...
    def __repr__(self) -> str: ...

    @overload
    def __getitem__(self, index: vec2i) -> T: ...
    @overload
    def __getitem__(self, index: tuple[int, int]) -> T: ...
    @overload
    def __getitem__(self, index: tuple[slice, slice]) -> array2d_view[T]: ...
    @overload
    def __getitem__(self, index: tuple[slice, int] | tuple[int, slice]) -> array2d_view[T]: ...
    @overload
    def __getitem__(self, mask: array2d_like[bool]) -> list[T]: ...
    @overload
    def __setitem__(self, index: vec2i, value: T): ...
    @overload
    def __setitem__(self, index: tuple[int, int], value: T): ...
    @overload
    def __setitem__(self, index: tuple[slice, slice], value: T | 'array2d_like[T]'): ...
    @overload
    def __setitem__(self, index: tuple[slice, int] | tuple[int, slice], value: T | 'array2d_like[T]'): ...
    @overload
    def __setitem__(self, mask: array2d_like[bool], value: T): ...

    # algorithms
    def count(self, value: T) -> int:
        """Count the number of cells with the given value."""

    def count_neighbors(self, value: T, neighborhood: Neighborhood) -> array2d[int]:
        """Count the number of neighbors with the given value for each cell."""

    def get_bounding_rect(self, value: T) -> tuple[int, int, int, int]:
        """Get the bounding rectangle of the given value.
        
        Returns a tuple `(x, y, width, height)` or raise `ValueError` if the value is not found.
        """

    def convolve(self: array2d_like[int], kernel: array2d_like[int], padding: int) -> array2d[int]:
        """Convolve the array with the given kernel."""

    def get_connected_components(self, value: T, neighborhood: Neighborhood) -> tuple[array2d[int], int]:
        """Get connected components of the grid via BFS algorithm.

        Returns the `visited` array and the number of connected components,
        where `0` means unvisited, and non-zero means the index of the connected component.
        """


class array2d_view[T](array2d_like[T]):
    @property
    def origin(self) -> vec2i: ...


class array2d[T](array2d_like[T]):
    def __new__(
            cls,
            n_cols: int,
            n_rows: int,
            default: T | Callable[[vec2i], T] | None = None
            ): ...

    @staticmethod
    def fromlist(data: list[list[T]]) -> array2d[T]: ...


class chunked_array2d[T, TContext]:
    def __new__(
            cls,
            chunk_size: int,
            default: T | None = None,
            auto_add_chunk: bool = True,
            ): ...
    
    @property
    def chunk_size(self) -> int: ...

    def __getitem__(self, index: vec2i) -> T: ...
    def __setitem__(self, index: vec2i, value: T): ...
    def __iter__(self) -> Iterator[tuple[vec2i, TContext]]: ...
    def __len__(self) -> int: ...

    def clear(self) -> None: ...
    def copy(self) -> Self: ...

    def world_to_chunk(self, world_pos: vec2i) -> tuple[vec2i, vec2i]:
        """Converts world position to chunk position and local position."""
        
    def add_chunk(self, chunk_pos: vec2i, context: TContext | None) -> None: ...
    def remove_chunk(self, chunk_pos: vec2i) -> bool: ...
    def move_chunk(self, src_chunk_pos: vec2i, dst_chunk_pos: vec2i) -> bool: ...
    def get_context(self, chunk_pos: vec2i) -> TContext | None: ...

    def view(self) -> array2d_view[T]: ...
    def view_rect(self, pos: vec2i, width: int, height: int) -> array2d_view[T]: ...
    def view_chunk(self, chunk_pos: vec2i) -> array2d_view[T]: ...
    def view_chunks(self, chunk_pos: vec2i, width: int, height: int) -> array2d_view[T]: ...



exit()

import colorcvt
from vmath import vec3

def oklch(expr: str) -> vec3:
    # oklch(82.33% 0.37 153)
    expr = expr[6:-1]
    l, c, h = expr.split()
    l = float(l[:-1]) / 100
    return vec3(l, float(c), float(h))

def srgb32(expr: str) -> vec3:
    # rgb(0, 239, 115)
    expr = expr[4:-1]
    r, g, b = expr.split(", ")
    r, g, b = int(r), int(g), int(b)
    c = vec3(r, g, b) / 255
    return colorcvt.srgb_to_linear_srgb(c)

def assert_equal(title: str, a: vec3, b: vec3) -> None:
    epsilon = 1e-3
    try:
        assert abs(a.x - b.x) < epsilon
        assert abs(a.y - b.y) < epsilon
        assert abs(a.z - b.z) < epsilon
    except AssertionError:
        raise AssertionError(f"{title}\nExpected: {b}, got: {a}")

def test(oklch_expr: str, srgb32_expr: str) -> None:
    oklch_color = oklch(oklch_expr)
    srgb32_color = srgb32(srgb32_expr)
    assert_equal('oklch_to_linear_srgb', colorcvt.oklch_to_linear_srgb(oklch_color), srgb32_color)
    # in range check
    oklch_color = colorcvt.linear_srgb_to_oklch(srgb32_color)
    assert_equal('oklch_to_linear_srgb+', colorcvt.oklch_to_linear_srgb(oklch_color), srgb32_color)
    assert_equal('linear_srgb_to_oklch+', colorcvt.linear_srgb_to_oklch(srgb32_color), oklch_color)

test("oklch(71.32% 0.1381 153)", "rgb(83, 187, 120)")
test("oklch(45.15% 0.037 153)", "rgb(70, 92, 76)")
test("oklch(22.5% 0.0518 153)", "rgb(4, 34, 16)")

test("oklch(100% 0.37 153)", "rgb(255, 255, 255)")
test("oklch(0% 0.0395 283.24)", "rgb(0, 0, 0)")

# hard samples
# test("oklch(95% 0.2911 264.18)", "rgb(224, 239, 255)")
# test("oklch(28.09% 0.2245 153)", "rgb(0, 54, 12)")
# test("oklch(82.33% 0.37 153)", "rgb(0, 239, 115)")



from .writer import Writer
from .converters import get_converter, Converter
from .schema import Struct, StructField

def gen_getter(w: Writer, struct: Struct, cvt: Converter, field: StructField):
    w.write(f'static bool {struct.identifier}__get_{field.name}(int argc, py_Ref argv) {{')
    w.indent()
    w.write(f'PY_CHECK_ARGC(1);')
    w.write(f'{struct.code_name}* self = py_touserdata(argv);')
    cvt.c2py(w, 'py_retval()', f'self->{field.name}')
    w.write('return true;')
    w.dedent()
    w.write('}')

def gen_setter(w: Writer, struct: Struct, cvt: Converter, field: StructField):
    w.write(f'static bool {struct.identifier}__set_{field.name}(int argc, py_Ref argv) {{')
    w.indent()
    w.write(f'PY_CHECK_ARGC(2);')
    w.write(f'{struct.code_name}* self = py_touserdata(argv);')
    cvt.py2c(w, f'self->{field.name}', 'py_arg(1)')
    w.write('py_newnone(py_retval());')
    w.write('return true;')
    w.dedent()
    w.write('}')

def gen_struct(w: Writer, pyi_w: Writer, struct: Struct):
    name = struct.code_name
    identifier = struct.identifier
    converters = [get_converter(field.type) for field in struct.fields or []]
    # default __new__
    w.write(f'static bool {identifier}__new__(int argc, py_Ref argv) {{')
    w.indent()
    w.write(f'py_Type cls = py_totype(argv);')
    w.write(f'py_newobject(py_retval(), cls, 0, sizeof({name}));')
    w.write('return true;')
    w.dedent()
    w.write('}')

    # default __init__
    w.write(f'static bool {identifier}__init__(int argc, py_Ref argv) {{')
    w.indent()
    w.write(f'{name}* self = py_touserdata(argv);')
    w.write(f'if(argc == 1) {{')
    w.indent()
    w.write(f'memset(self, 0, sizeof({name}));')
    w.dedent()
    w.write(f'}} else if(argc == 1 + {len(struct.fields)}) {{')
    w.indent()
    for i, field in enumerate(struct.fields):
        cvt = converters[i]
        if not cvt.is_const():
            cvt.py2c(w, f'self->{field.name}', f'py_arg({i+1})')
        else:
            w.write(f'// _{i} {field.name} is read-only')
    w.dedent()
    w.write('} else {')
    w.indent()
    w.write(f'return TypeError("expected 1 or {len(struct.fields)+1} arguments");')
    w.dedent()
    w.write('}')
    w.write('py_newnone(py_retval());')
    w.write('return true;')
    w.dedent()
    w.write('}')

    for i, field in enumerate(struct.fields):
        cvt = converters[i]
        gen_getter(w, struct, cvt, field)
        if not cvt.is_const():
            gen_setter(w, struct, cvt, field)

    w.write(f'static py_Type register__{identifier}(py_GlobalRef mod) {{')
    w.indent()
    w.write(f'py_Type type = py_newtype("{identifier}", tp_stdc_Memory, mod, NULL);')
    w.write(f'py_newint(py_emplacedict(py_tpobject(type), py_name("size")), sizeof({name}));')

    w.write(f'py_bindmethod(type, "__new__", {identifier}__new__);')
    w.write(f'py_bindmethod(type, "__init__", {identifier}__init__);')

    for i, field in enumerate(struct.fields):
        cvt = converters[i]
        if cvt.is_const():
            setter = 'NULL'
        else:
            setter = f'{identifier}__set_{field.name}'
        w.write(f'py_bindproperty(type, "{field.name}", {identifier}__get_{field.name}, {setter});')

    w.write(f'return type;')
    w.dedent()
    w.write('}')

    # pyi
    pyi_w.write(f'class {identifier}:')
    pyi_w.indent()

    py_args = []
    for i, field in enumerate(struct.fields):
        cvt = converters[i]
        desc = (field.desc or '') + f' ({field.type})'
        py_args.append(f"{field.name}: {cvt.py_T}")
        pyi_w.write(f"{py_args[-1]} # {desc}")
    pyi_w.write('')

    pyi_w.write(f'@overload')
    pyi_w.write(f'def __init__(self): ...')
    pyi_w.write(f'@overload')
    pyi_w.write(f'def __init__(self, {", ".join(py_args)}): ...')
    pyi_w.write('')
    pyi_w.dedent()

    w.write(f'static py_Type tp_user_{identifier};')
    return f'tp_user_{identifier} = register__{identifier}(mod);'


# multi-loop generator
out = []
args = map(lambda x: x+1, [1, 2, 3])
for x in args:
    for y in args:
        out.append((x, y))
assert out == [(2, 3), (2, 4)]

# multi loop bug
out = []
a = [1, 2]
for i in a:
    for j in a:
        out.append((i, j))
assert (out == [(1, 1), (1, 2), (2, 1), (2, 2)]), out

# https://github.com/pocketpy/pocketpy/issues/37

mp = map(lambda x:  x**2, [1, 2, 3, 4, 5]  )
assert list(mp) == [1, 4, 9, 16, 25]


assert not 3>4

def f(x):
    if x>1:
        return 1

assert f(2) == 1
assert f(0) == None

a = [1, 2]
b = [3, 4]
assert a.append == a.append
assert a.append is not a.append
assert a.append is not b.append
assert a.append != b.append

inq = 0
if not inq:
    assert True
else:
    assert False

a = object()
b = object()
if a is   not b:
    assert True
if a  ==  0:
    assert False

def g(x):
    return x
def f(x):
    return x

assert (g(1), 2) == (1, 2)
assert (
    g(1),
    2
) == (1, 2)

assert f((
    g(1),
    2
)) == (1, 2)

def f():
    for i in range(4):
        _ = 0
    while i: i-=1
f()

# class A: a=b=1
# class A: a, b = 1, 2

bmi = 0.0

def test(a):
    if a:
        bmi = 1.4
    return f'{bmi:.2f}'

assert test(1) == '1.40'

try:
    x = test(0)
    print('x:', x)
    exit(1)
except UnboundLocalError:
    pass


g = 1
def f():
    global g
    g += 1

f(); f()
assert g == 3


def f(**kw):
    x = 1
    y = 2
    return kw, x, y
assert f(x=4, z=1) == ({'x': 4, 'z': 1}, 1, 2)

def g(**kw):
    x, y = 1, 2
    return kw

ret = g(
    a=1, b=2, c=3, d=4, e=5, f=6, g=7, h=8, i=9,
    j=10, k=11, l=12, m=13, n=14, o=15, p=16, q=17,
    r=18, s=19, t=20, u=21, v=22, w=23, x=24, y=25,
    z=26
)
assert ret == {chr(i+97): i+1 for i in range(26)}

assert g(**ret) == ret
assert g(**g(**ret)) == ret

# other known issues:
# 1. d.extend(d) if d is deque

g = 0
def test():
    global g
    g += 1
    return g

a = [1, 10, 3]
a[test()] += 1
assert (a == [1, 11, 3]), a
assert (g == 1), g

assert list.__new__(list) == []
assert a.__new__ == list.__new__


class A:
    x: list[int] = [i for i in range(1, 4)]

assert A.x == [1, 2, 3]

# stable sort
a = [(0, 1), (1, 1), (1, 2)]
b = sorted(a, key=lambda x: x[0])
if b != [(0, 1), (1, 1), (1, 2)]:
    assert False, b

# https://github.com/pocketpy/pocketpy/issues/367
a = 10 if False else 5
assert a == 5

a, b, c = (1, 2, 3) if True else (4, 5, 6)
assert a == 1
assert b == 2
assert c == 3

# https://github.com/pocketpy/pocketpy/issues/376
xs = [0]
res = []
for x in xs:
    res.append(x)
    if x == 100:
        break
    xs.append(x+1)

assert res == list(range(101))
assert xs == res

# call property
from vmath import vec2

a = vec2(1, 2)
try:
    x = a.x()
    exit(1)
except TypeError:
    pass


def g():
    yield 1
    yield 2
    yield

a = g()
assert next(a) == 1
assert next(a, None) == 2
assert next(a) == None

try:
    next(a)
    exit(1)
except StopIteration:
    pass

assert next(a, 3) == 3
assert next(a, 4) == 4

def f(n):
    for i in range(n):
        yield i

x = 0
for j in f(5):
    x += j

assert x == 10

a = [i for i in f(6)]

assert a == [0,1,2,3,4,5]

def f(n):
    for i in range(n):
        for j in range(n):
            yield i, j

a = [i for i in f(3)]
assert len(a) == 9
assert a[0] == (0,0)
assert a[1] == (0,1)
assert a[2] == (0,2)
assert a[3] == (1,0)
assert a[4] == (1,1)
assert a[5] == (1,2)
assert a[6] == (2,0)
assert a[7] == (2,1)
assert a[8] == (2,2)

def g():
    yield from [1, 2, 3]

def f():
    yield from g()

a = [i for i in f()]
assert a == [1, 2, 3]


def f():
    for i in range(5):
        yield str(i)
assert '|'.join(f()) == '0|1|2|3|4'


def f(n):
    for i in range(n):
        yield i
        for j in range(i):
            yield j

t = f(3)
assert list(t) == [0, 1, 0, 2, 0, 1]

def f(n):
    for i in range(n):
        if i == n-1:
            raise ValueError
        yield i

t = f(3)
t = iter(t)
assert next(t) == 0
assert next(t) == 1

try:
    next(t)
    exit(1)
except ValueError:
    pass

try:
    next(t)
    exit(1)
except StopIteration:
    pass

def f():
    yield 1
    yield 2
    return
    yield 3

assert list(f()) == [1, 2]

def g():
    yield 1
    yield 2
    return 3
    yield 4

assert StopIteration().value == None
assert StopIteration(3).value == 3

try:
    iter = g()
    assert next(iter) == 1
    assert next(iter) == 2
    next(iter)  # raises StopIteration
    print('UNREACHABLE!!')
    exit(1)
except StopIteration as e:
    assert e.value == 3

def f():
    a = yield from g()
    yield a

assert list(f()) == [1, 2, 3]


try:
    a = [1, 2, 3]
    a.index(999)
    exit(1)
except ValueError:
    pass

# test some python magics
class TestMagics:
    def __init__(self):
        self.d = {}

    def __getitem__(self, index):
        return self.d[index]
    
    def __setitem__(self, index, value):
        self.d[index] = value

    def __contains__(self, index):
        return index in self.d
    
    def __delitem__(self, index):
        del self.d[index]

a = TestMagics()
a['1'] = 3
assert '1' in a
assert '2' not in a
assert a['1'] == 3
del a['1']
assert '1' not in a

# slice extras
class A:
    def __getitem__(self, index):
        return index

assert slice(1, 2, None) == slice(1, 2, None)
assert slice(1, 3, None) != slice(1, 2, None)

assert A()[1] == 1
assert A()[1:2, 3] == (slice(1, 2, None), 3)
assert A()[1:2, 3:4] == (slice(1, 2, None), slice(3, 4, None))
assert A()[1:2, 3:4, 5] == (slice(1, 2, None), slice(3, 4, None), 5)
assert A()[:, :] == (slice(None, None, None), slice(None, None, None))
assert A()[::, :] == (slice(None, None, None), slice(None, None, None))
assert A()[::, :2] == (slice(None, None, None), slice(None, 2, None))
assert A()['b':'c':1, :] == (slice('b', 'c', 1), slice(None, None, None))
assert A()[1:2, :A()[3:4, ::-1]] == (slice(1, 2, None), slice(None, (slice(3, 4, None), slice(None, None, -1)), None))

# test right associative
assert 2**2**3 == 256
assert (2**2**3)**2 == 65536

class Number:        
    def __divmod__(self, other):
        return 3, 4
    
    def __round__(self, *args):
        return args

assert divmod(Number(), 0) == (3, 4)
assert round(Number()) == tuple()
assert round(Number(), 1) == (1,)

class Z:
    def __new__(cls, x):
        return cls, x

class B(Z):
    def __new__(cls, x):
        assert super() is Z
        return super().__new__(cls, x)

assert Z(1) == (Z, 1)
assert B(1) == (B, 1)

from pkpy import TValue

class fixed(TValue[int]):
    def __new__(cls, value: str):
        assert super() is TValue[int]
        return super().__new__(cls, int(value))
    
assert fixed('123').value == 123

# context bug
class Context:
  def __enter__(self):
    return 1
  def __exit__(self, *_):
    pass

for _ in range(5):
    with Context() as x:
        assert x == 1

# bad dict hash
class A:
    def __eq__(self, o): return False
    def __ne__(self, o): return True
    def __hash__(self): return 1

bad_dict = {A(): 1, A(): 2, A(): 3, A(): 4}
assert len(bad_dict) == 4




# https://github.com/pocketpy/pocketpy/issues/456

module_code = '''
CONSTANT = 42

def hello(name):
    return "Hello, " + name
'''

namespace = {}

exec(module_code, namespace)

assert namespace['CONSTANT'] == 42
assert namespace['hello']('world') == "Hello, world"
# print("Constant:", namespace['CONSTANT'])
# print("Function result:", namespace['hello']('world'))



import os
import sys

def check_pragma_once_in_dir(path):
    for root, dirs, files in os.walk(path):
        if 'include/pocketpy/xmacros' in root or 'include/pybind11' in root:
            continue
        for file in files:
            if file.endswith(".c") or file.endswith(".h"):
                with open(os.path.join(root, file), "r", encoding='utf-8') as f:
                    print(f"==> {os.path.join(root, file)}")
                    text = f.read()
                    assert text.count("#pragma once") == 1, "#pragma once should appear exactly once"

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python scripts/check_pragma_once.py <path>")
        sys.exit(1)

    check_pragma_once_in_dir(sys.argv[1])


from dataclasses import dataclass, asdict

@dataclass
class A:
    x: int
    y: str = '123'

assert repr(A(1)) == "A(x=1, y='123')"
assert repr(A(x=3)) == "A(x=3, y='123')"
assert repr(A(1, '555')) == "A(x=1, y='555')"
assert repr(A(x=7, y='555')) == "A(x=7, y='555')"

assert asdict(A(1, '555')) == {'x': 1, 'y': '555'}

assert A(1, 'N') == A(1, 'N')
assert A(1, 'N') != A(1, 'M')

#################

@dataclass
class Base:
  i: int
  j: int

class Derived(Base):
  k: str = 'default'

  def sum(self):
    return self.i + self.j

d = Derived(1, 2)

assert d.i == 1
assert d.j == 2
assert d.k == 'default'
assert d.sum() == 3


@dataclass
class PrimaryForceConfig:
    # 风场图
    planetary_wind: 'str'
    local_wind: int
    # 地壳运动（含地震带/地形生成）
    geothermal_activity: int
    # 太阳辐射标量场
    solar_radiation: int
    # 水汽场
    planetary_humidity: int

config = PrimaryForceConfig(
   planetary_wind = 'default',
   local_wind = 1,
   geothermal_activity = 2,
   solar_radiation = 3,
   planetary_humidity = 4
)

assert config.planetary_wind == 'default'


# Heap queue algorithm (a.k.a. priority queue)
def heappush(heap, item):
    """Push item onto heap, maintaining the heap invariant."""
    heap.append(item)
    _siftdown(heap, 0, len(heap)-1)

def heappop(heap):
    """Pop the smallest item off the heap, maintaining the heap invariant."""
    lastelt = heap.pop()    # raises appropriate IndexError if heap is empty
    if heap:
        returnitem = heap[0]
        heap[0] = lastelt
        _siftup(heap, 0)
        return returnitem
    return lastelt

def heapreplace(heap, item):
    """Pop and return the current smallest value, and add the new item.

    This is more efficient than heappop() followed by heappush(), and can be
    more appropriate when using a fixed-size heap.  Note that the value
    returned may be larger than item!  That constrains reasonable uses of
    this routine unless written as part of a conditional replacement:

        if item > heap[0]:
            item = heapreplace(heap, item)
    """
    returnitem = heap[0]    # raises appropriate IndexError if heap is empty
    heap[0] = item
    _siftup(heap, 0)
    return returnitem

def heappushpop(heap, item):
    """Fast version of a heappush followed by a heappop."""
    if heap and heap[0] < item:
        item, heap[0] = heap[0], item
        _siftup(heap, 0)
    return item

def heapify(x):
    """Transform list into a heap, in-place, in O(len(x)) time."""
    n = len(x)
    # Transform bottom-up.  The largest index there's any point to looking at
    # is the largest with a child index in-range, so must have 2*i + 1 < n,
    # or i < (n-1)/2.  If n is even = 2*j, this is (2*j-1)/2 = j-1/2 so
    # j-1 is the largest, which is n//2 - 1.  If n is odd = 2*j+1, this is
    # (2*j+1-1)/2 = j so j-1 is the largest, and that's again n//2-1.
    for i in reversed(range(n//2)):
        _siftup(x, i)

# 'heap' is a heap at all indices >= startpos, except possibly for pos.  pos
# is the index of a leaf with a possibly out-of-order value.  Restore the
# heap invariant.
def _siftdown(heap, startpos, pos):
    newitem = heap[pos]
    # Follow the path to the root, moving parents down until finding a place
    # newitem fits.
    while pos > startpos:
        parentpos = (pos - 1) >> 1
        parent = heap[parentpos]
        if newitem < parent:
            heap[pos] = parent
            pos = parentpos
            continue
        break
    heap[pos] = newitem

def _siftup(heap, pos):
    endpos = len(heap)
    startpos = pos
    newitem = heap[pos]
    # Bubble up the smaller child until hitting a leaf.
    childpos = 2*pos + 1    # leftmost child position
    while childpos < endpos:
        # Set childpos to index of smaller child.
        rightpos = childpos + 1
        if rightpos < endpos and not heap[childpos] < heap[rightpos]:
            childpos = rightpos
        # Move the smaller child up.
        heap[pos] = heap[childpos]
        pos = childpos
        childpos = 2*pos + 1
    # The leaf at pos is empty now.  Put newitem there, and bubble it up
    # to its final resting place (by sifting its parents down).
    heap[pos] = newitem
    _siftdown(heap, startpos, pos)


a = 1 + 2 \
    + 3

assert a == 6

assert 1 + 2 \
    + 3 == 6

assert 1 + 2 + \
    3 + \
    4 == 10

assert 1 + 2 + \
    3 + \
    4 + 5 + 6 \
    == 21

if 1 and 2 \
    and 3 \
    and 4 \
    and 5:
    assert True
else:
    assert False

1 and 2 \
    and 3 \
    and 4

a = 1
assert a == 1

# https://github.com/pocketpy/pocketpy/issues/408
x = (
1
+
2
)

# https://peps.python.org/pep-0008/#should-a-line-break-before-or-after-a-binary-operator
compile('''
income = (gross_wages
          + taxable_interest
          + (dividends - qualified_dividends)
          - ira_deduction
          - student_loan_interest)
'''.strip(), 'main.py', 'exec')

res = []
class Thing:
	def do_a(self):
		res.append("A")
		return self
	def do_b(self):
		res.append("B")
		return self
	def do_c(self):
		res.append("C")
		return self
		
(Thing()
	.do_a()
	.do_b()
	.do_c())

assert res == ["A", "B", "C"]



from typing import Dict, Tuple, List

bill: Dict[str, float] = {
    "apple": 3.14,
    "watermelon": 15.92,
    "pineapple": 6.53,
}
completed: Tuple[str] = ("DONE",)
succeeded: Tuple[int, str] = (1, "SUCCESS")
statuses: Tuple[str, ...] = (
    "DONE", "SUCCESS", "FAILED", "ERROR",
)
codes: List[int] = (0, 1, -1, -2)


from typing import Union

def resp200(meaningful) -> Union[int, str]:
    return "OK" if meaningful else 200


from typing import Self

class Employee:
    name: str = "John Doe"
    age: int = 0

    def set_name(self: Self, name) -> Self:
        self.name = name
        return self
    
from typing import TypeVar, Type

T = TypeVar("T")

# "mapper" is a type, like int, str, MyClass and so on.
# "default" is an instance of type T, such as 314, "string", MyClass() and so on.
# returned is an instance of type T too.
def converter(raw, mapper: Type[T] = None, default: T = None) -> T:
    try:
        return mapper(raw)
    except:
        pass
    return default

raw: str = '4'
result: int = converter(raw, mapper=int, default=0)

from typing import TypeVar, Callable, Any

T = TypeVar("T")

def converter(raw, mapper: Callable[[Any], T] = None, default: T = None) -> T:
    try:
        return mapper(raw)
    except:
        pass
    return default

def is_success(value) -> bool:
    return value in (0, "OK", True, "success")

resp = {'code': 0, 'message': 'OK', 'data': []}
successed: bool = converter(resp['message'], mapper=is_success, default=False)


class A:
    x: List[Callable[[int], Any]]
    y: Dict[str, int]

a = A()
assert not hasattr(a, 'x')
assert not hasattr(a, 'y')

class B:
    x: List[Callable[[int], Any]] = []
    y: Dict[str, int] = {}

b = B()
assert hasattr(b, 'x')
assert hasattr(b, 'y')

abc123: int
assert 'abc123' not in globals()


from vmath import color32, rgb, rgba, vec3i

a = color32(100, 200, 255, 120)
assert a.r == 100
assert a.g == 200
assert a.b == 255
assert a.a == 120
assert a.with_r(255).r == 255 
assert a.with_g(255).g == 255
assert a.with_b(255).b == 255
assert a.with_a(255).a == 255 and a.with_a(255).g == a.g

assert a.to_hex() == '#64c8ff78'
assert color32.from_hex('#64c8ff78') == a
assert color32.from_hex('#64c8ff') == a.with_a(255)

assert rgb(100, 200, 255) != a
assert rgba(100, 200, 255, 120 / 255) == a

b = color32(75, 150, 200, 200)
assert a == a and b == b
assert a != b

assert repr(b) == 'color32(75, 150, 200, 200)'

assert color32.from_vec3i(vec3i(100, 200, 255)) == rgb(100, 200, 255)
alpha = a.a / 255
assert a.to_vec3i() == vec3i(int(a.r * alpha), int(a.g * alpha), int(a.b * alpha))

# assert color32.alpha_blend(a, b) == color32(86, 173, 225, 162)

c = rgb(100, 200, 255)
assert c.ansi_fg('xxx') == '\x1b[38;2;100;200;255mxxx\x1b[0m'
assert c.ansi_bg('xxx') == '\x1b[48;2;100;200;255mxxx\x1b[0m'

assert color32.alpha_blend(a, None) == a



from dis import dis

def f(a):
    for i in range(100000):
        a.append(i)
    a = 0.5
    a = True
    a = '123'
    a = 123, 456
    a = [1, 2, 3]
    return a, a, a

def g(a):
    return f([1,2,3] + a)

assert dis(f) is None



assert 'testing' == 'test' + 'ing'
assert 'testing' != 'test' + 'ing2'
assert 'testing' < 'test' + 'ing2'
assert 'testing' <= 'test' + 'ing2'
assert 'testing5' > 'test' + 'ing1'
assert 'testing5' >= 'test' + 'ing1'

# test + *=
assert 'abc' + 'def' == 'abcdef'
assert 'abc' * 3 == 'abcabcabc'

assert repr('\\\n\t\'\r\b\x48') in [
    r"'\\\n\t\'\r\bH'",
    '"\\\\\\n\\t\'\\r\\x08H"',
]

a = ''
b = 'test'
c ='test'
assert a == str()
assert len(a) == 0
assert len(b) == 4
assert b == c

# upper and lower does not work for utf-8
assert ''.lower() == '' and ''.upper() == ''
assert 'already+lower '.lower() == 'already+lower '
assert 'ALREADY+UPPER '.upper() == 'ALREADY+UPPER '
assert 'tEST+InG'.lower() == 'test+ing'
assert 'tEST+InG'.upper() == 'TEST+ING'

s = "football"
q = "abcd"
r = "zoo"
t = "this is string example....wow!!!"
assert s[0] == 'f'
assert s[1:4] == 'oot'
assert s[:-1] == 'footbal'
assert s[:10] == 'football'
assert s[-3] == 'a'
assert t[-5:] == 'ow!!!'
assert t[3:-3] == 's is string example....wow'
assert s > q;assert s < r
assert s.replace("o","") == "ftball"
assert s.replace("o","O") == "fOOtball"
assert s.replace("foo","ball") == "balltball"
assert s.startswith('f') == True;assert s.endswith('o') == False
assert t.startswith('this') == True;

assert t.split('w') == ['this is string example....', 'o', '!!!']
assert "a,b,c".split(',') == ['a', 'b', 'c']
assert 'a,'.split(',') == ['a', '']
assert 'foo!!bar!!baz'.split('!!') == ['foo', 'bar', 'baz']
assert ' 4 3 '.split() == ['4', '3']
assert '  4 3  '.split(' ') == ['', '', '4', '3', '', '']
assert 'aa bb cccc'.split('cc') == ['aa bb ', '', '']
assert '.a.b.'.split('.') == ['', 'a', 'b', '']
assert '.a...b.'.split('.') == ['', 'a', '', '', 'b', '']

# https://github.com/pocketpy/pocketpy/issues/378
assert "a b   \n   c\td".split() == ['a', 'b', 'c', 'd']

try:
    'a'.split('')
    exit(1)
except ValueError:
    pass

assert '111'.count('1') == 3
assert '111'.count('11') == 1
assert '1111'.count('11') == 2
assert '11'.count('') == 3
assert ''.count('1') == 0
assert ''.count('') == 1

t = "*****this is **string** example....wow!!!*****"
s = "123abcrunoob321"
assert t.strip( '*' ) == "this is **string** example....wow!!!"
assert s.strip( '12' ) == "3abcrunoob3"

assert t.strip( '*' ) == "this is **string** example....wow!!!"
assert s.strip( '12' ) == "3abcrunoob3"

assert '测试123'.strip('测试') == '123'
assert '测试123测试'.strip('测试') == '123'
assert '123测试'.strip('2') == '123测试'
assert '测试123'.strip('测') == '试123'
assert '测试123'.strip('试') == '测试123'

assert '测试123测试'.lstrip('测试') == '123测试'
assert '测试123测试'.rstrip('测试') == '测试123'

assert 'abc'.lstrip('a') == 'bc'
assert 'abc'.lstrip('b') == 'abc'
assert 'abc'.lstrip('c') == 'abc'
assert 'abc'.rstrip('a') == 'abc'
assert 'abc'.rstrip('b') == 'abc'
assert 'abc'.rstrip('c') == 'ab'

assert 'abc'.lstrip('abc') == ''
assert 'abc'.rstrip('abc') == ''
assert 'abc'.strip('abc') == ''

s = ' asd\n  asd \n'
assert s.strip() == 'asd\n  asd'

s1 = "-"
s2 = ""
seq = ["r","u","n","o","o","b"]
assert s1.join( seq ) == "r-u-n-o-o-b"
assert s2.join( seq ) == "runoob"

assert 'x'.zfill(5) == '0000x'
assert '568'.zfill(1) == '568'

num = 6
assert str(num) == '6'

# test Lo group names
测试 = "test"
assert 测试 == "test"

a = 'abcd'
assert list(a) == ['a', 'b', 'c', 'd']
a = '测试'
assert list(a) == ['测', '试']
a = 'a测b试c'
assert list(a) == ['a', '测', 'b', '试', 'c']
a = 'a测b试'
assert list(a) == ['a', '测', 'b', '试']
a = '测b试c'
assert list(a) == ['测', 'b', '试', 'c']
a = '测b'
assert list(a) == ['测', 'b']
a = 'b'
assert list(a) == ['b']
a = '测'
assert list(a) == ['测']

# 3rd slice
a = "Hello, World!"
assert a[::-1] == "!dlroW ,olleH"
assert a[::2] == "Hlo ol!"
assert a[2:5:2] == "lo"
assert a[5:2:-1] == ",ol"
assert a[5:2:-2] == ",l"

a = '123'
assert a.rjust(5) == '  123'
assert a.rjust(5, '0') == '00123'
assert a.ljust(5) == '123  '
assert a.ljust(5, '0') == '12300'

assert '\x30\x31\x32' == '012'
assert '\b\b\b' == '\x08\x08\x08'
assert repr('\x1f\x1e\x1f') == '\'\\x1f\\x1e\\x1f\''

assert '\a' == '\x07'
assert '\b' == '\x08'
assert '\f' == '\x0c'
assert '\v' == '\x0b'
assert '\0' == '\x00'

a = '123'
assert a.index('2') == 1
assert a.index('1') == 0
assert a.index('3') == 2
assert a.index('23') == 1

assert a.index('2', 1) == 1
assert a.index('1', 0) == 0
assert a.index('2', -2) == 1

assert a.find('1') == 0
assert a.find('1', 1) == -1

b = list("Hello, World!")
assert b == ['H', 'e', 'l', 'l', 'o', ',', ' ', 'W', 'o', 'r', 'l', 'd', '!']
assert b[::-1] == ['!', 'd', 'l', 'r', 'o', 'W', ' ', ',', 'o', 'l', 'l', 'e', 'H']
assert b[::2] == ['H', 'l', 'o', ' ', 'o', 'l', '!']
assert b[2:5:2] == ['l', 'o']
assert b[5:2:-1] == [',', 'o', 'l']
assert b[5:2:-2] == [',', 'l']

assert hex(-42) == '-0x2a'
assert hex(42) == '0x2a'

assert hex(0) == '0x0'
assert hex(1) == '0x1'
assert hex(15) == '0xf'
assert hex(16) == '0x10'
assert hex(255) == '0xff'
assert hex(256) == '0x100'
assert hex(257) == '0x101'
assert hex(17) == '0x11'

assert bin(-42) == '-0b101010'
assert bin(42) == '0b101010'

assert bin(0) == '0b0'
assert bin(1) == '0b1'
assert bin(15) == '0b1111'
assert bin(16) == '0b10000'
assert bin(255) == '0b11111111'
assert bin(256) == '0b100000000'
assert bin(257) == '0b100000001'
assert bin(17) == '0b10001'

assert '-'.join(['r', 'u', 'n', 'o', 'o', 'b']) == 'r-u-n-o-o-b'

assert (1 != '1') is True
assert (1 == '1') is False
assert 1 == 1.0

assert chr(97) == 'a'
assert ord('a') == 97

assert ord('🥕') == 0x1f955
assert chr(0x1f955) == '🥕'

assert ord('测') == 27979
assert chr(27979) == '测'

# test format()
assert "Hello, {}!".format("World") == "Hello, World!"
assert "{} {} {}".format("I", "love", "Python") == "I love Python"
assert "{0} {1} {2}".format("I", "love", "Python") == "I love Python"
assert "{2} {1} {0}".format("I", "love", "Python") == "Python love I"
assert "{0}{1}{0}".format("abra", "cad") == "abracadabra"

# assert "{k}={v}".format(k="key", v="value") == "key=value"
# assert "{k}={k}".format(k="key") == "key=key"
assert "{0}={1}".format('{0}', '{1}') == "{0}={1}"
assert "{{{0}}}".format(1) == "{1}"
assert "{0}{1}{1}".format(1, 2, 3) == "122"

try:
    "{0}={1}}".format(1, 2)
    exit(1)
except ValueError:
    pass

assert "{{{}xxx{}x}}".format(1, 2) == "{1xxx2x}"
assert "{{abc}}".format() == "{abc}"

# test f-string
assert f"{1+2}" == "3"
# assert f"{1, 2, 3}" == "(1, 2, 3)"
assert f"{(1, 2, 3)}" == "(1, 2, 3)"

# stack=[1,2,3,4]
# assert f"{stack[2:]}" == '[3, 4]'


assert id('1' * 16) is not None
assert id('1' * 15) is None


import re

whitelist = {'K', 'equal', 'V', 'less', 'NAME'}

def check_define_undef_pairs(code):
    # 使用正则表达式匹配#define和#undef指令
    define_pattern = re.compile(r'#define\s+(\w+)')
    undef_pattern = re.compile(r'#undef\s+(\w+)')

    # 查找所有的#define和#undef
    defines = define_pattern.findall(code)
    undefs = undef_pattern.findall(code)

    defines = [x for x in defines if x not in whitelist]
    undefs = [x for x in undefs if x not in whitelist]

    # 使用集合计算差集，找出不匹配的部分
    define_set = set(defines)
    undef_set = set(undefs)

    unmatched_defines = define_set - undef_set
    unmatched_undefs = undef_set - define_set

    if unmatched_defines or unmatched_undefs:
        if unmatched_defines:
            print("mismatched #define")
            for define in unmatched_defines:
                print(f"- {define}")
            exit(1)

        if unmatched_undefs:
            print("mismatched #undef")
            for undef in unmatched_undefs:
                print(f"- {undef}")
            exit(1)


# iterate over all the files in `path` directory

import os
import sys

def check_undef_in_dir(path):
    for root, dirs, files in os.walk(path):
        for file in files:
            if file.endswith(".c") or file.endswith(".h"):
                with open(os.path.join(root, file), "r", encoding='utf-8') as f:
                    print(f"==> {os.path.join(root, file)}")
                    check_define_undef_pairs(f.read())

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: python scripts/check_undef.py <path>")
        sys.exit(1)

    check_undef_in_dir(sys.argv[1])


# if tests
flag = False
name = 'luren'
if name == 'python':         
    flag = True              
else:
    flag               
assert flag == False


num = 9
flag = 0
if num >= 0 and num <= 10:    
    flag = 1
else:
    flag
assert flag == 1

num = 10
flag = 0
if num < 0 or num > 10:    
    flag = 1
else:
    flag
assert flag == 0

num = 5
result = 0     
if num == 3:            
    result = num        
elif num == 2:
    result = num
elif num == 1:
    result = num
elif num < 0:           
    result = num
else:
    result = num
assert result == 5    

# for tests 

k = 0
for i in range(2, 1000):
  if i % 2 == 0:
    k += 1
assert k ==499

k = 0
for x in range(100):
    if x<2:
      continue
    flag = True
    for i in range(2,x):
      if x%i == 0:
        flag = False
        break
    if flag:
      k += 1
assert k == 25

#while tests
count = 0
while (count < 1000):
   count = count + 1
assert count == 1000

# ternary operator
d = 1 if 2 > 1 else 2
assert d == 1
d = 1 if 2 < 1 else 2
assert d == 2

t = 0
for i in range(5):
    if True and 1:
        break
    t = 1
assert t == 0

for i in range(5):
    break
else:
    assert False

for i in range(5):
    if i==3:
        break
else:
    assert False

flag = False
for i in range(5):
    if i==6:
        break
else:
    flag = True
assert flag is True

while True:
    break
else:
    assert False

flag = False
while False:
    assert False
else:
    flag = True
assert flag is True

x = 1
while 0:
    while True:
        break
else:
    x = 2
assert x == 2

if x == 2:
    while 0:
        pass
else:
    x = 3
assert x == 2

# match case
case, _1 = 1, 2
assert case == 1 and _1 == 2

match (404 * 1):
    case 200:
        assert False
    case 404:
        assert True
    case _: assert False

match (555 * 1):
    case 200:
        assert False
    case 404: assert False
    case _:
        assert True

match (555 * 1):
    case 200:
        assert False
    case 404:
        assert False
    # no default case


def f(case):
    match case:
        case 200:
            return True
        case 404:
            return False
        case _:
            return False

assert f(200) == True

# extras

t = 0
for i in range(5):
    try:
        break
    except:
        pass
    t = 1
assert t == 0



import re
from dataclasses import dataclass

NEWLINE = '\n'

@dataclass
class Function:
    name: str
    args: str
    ret: str
    comment: str
    is_py_raise: bool
    is_py_return: bool

    def signature(self):
        tmp = f"PK_API {self.ret} {self.name}{self.args}"
        return tmp + ';'
    
    def badges(self):
        res = []
        if self.is_py_raise:
            res.append('[!badge text="raise" variant="danger"](../introduction/#py_raise-macro)')
        if self.is_py_return:
            res.append('[!badge text="return"](../introduction/#py_return-macro)')
        return ' '.join(res)

    def markdown(self):
        lines = [
            f"### {self.name}" + f" {self.badges()}",
            f"```c",
            self.comment,
            f"{self.signature()}",
            f"```",
        ]
        return '\n'.join(lines)

with open('include/pocketpy/pocketpy.h') as f:
    header = f.read()

matches = re.finditer(r"((?:/// [^\n]+[\n])*?)PK_API\s+(\w+\*?)\s+(\w+)(\(.*?\))\s*(PY_RAISE)?\s*(PY_RETURN)?\s*;", header, re.DOTALL)
#                       ^1 comment                        ^2 ret     ^3 n ^4 args     ^5 py_raise?  ^6 py_return?

functions: list[Function] = []
for match in matches:
    functions.append(Function(
        name=match[3],
        args=match[4],
        ret=match[2],
        comment=match[1].strip(),
        is_py_raise=bool(match[5]),
        is_py_return=bool(match[6])
    ))
    # print(functions[-1])


# generate markdown
with open('docs/C-API/functions.md', 'w', newline='\n') as f:
    f.write('\n'.join([
        '---',
        'title: Functions',
        'icon: dot',
        'order: 0',
        '---',
        '\n\n',
    ]))
    for function in functions:
        f.write(function.markdown())
        f.write('\n\n')


class _Placeholder:
    def __init__(self, *args, **kwargs):
        pass
    def __getitem__(self, *args):
        return self
    def __call__(self, *args, **kwargs):
        return self
    def __and__(self, other):
        return self
    def __or__(self, other):
        return self
    def __xor__(self, other):
        return self


_PLACEHOLDER = _Placeholder()

Sequence = _PLACEHOLDER
List = _PLACEHOLDER
Dict = _PLACEHOLDER
Tuple = _PLACEHOLDER
Set = _PLACEHOLDER
Any = _PLACEHOLDER
Union = _PLACEHOLDER
Optional = _PLACEHOLDER
Callable = _PLACEHOLDER
Type = _PLACEHOLDER
TypeAlias = _PLACEHOLDER
NewType = _PLACEHOLDER

ClassVar = _PLACEHOLDER

Literal = _PLACEHOLDER
LiteralString = _PLACEHOLDER

Iterable = _PLACEHOLDER
Generator = _PLACEHOLDER
Iterator = _PLACEHOLDER

Hashable = _PLACEHOLDER

TypeVar = _PLACEHOLDER
Self = _PLACEHOLDER

Protocol = object
Generic = object
Never = object

TYPE_CHECKING = False

# decorators
overload = lambda x: x
final = lambda x: x

# exhaustiveness checking
assert_never = lambda x: x

TypedDict = dict
NotRequired = _PLACEHOLDER

cast = lambda _, val: val



try:
    import os
except ImportError:
    exit(0)

import importlib

os.chdir('tests')
assert os.getcwd().endswith('tests')

# test
os.environ['TEST_RELOAD_VALUE'] = '123'
os.environ['SET_X'] = '1'
os.environ['SET_Y'] = '0'

from testreload import MyClass, a

objid = id(MyClass)
funcid = id(MyClass.some_func)
getxyid = id(MyClass.get_xy)

assert MyClass.value == '123'
assert MyClass.get_xy() == (1, 0)

inst = MyClass()
assert inst.some_func() == '123'

# reload
os.environ['TEST_RELOAD_VALUE'] = '456'
os.environ['SET_X'] = '0'
os.environ['SET_Y'] = '1'

importlib.reload(a)

assert id(MyClass) == objid
assert id(MyClass.some_func) != funcid
assert id(MyClass.get_xy) != getxyid

assert MyClass.value == '456'
assert inst.some_func() == '456'
assert (MyClass.get_xy() == (1, 1)), MyClass.get_xy()


